@@ -58,6 +58,11 @@ function demangleCSymbolName(f) {
   return f[0] == '_' ? f.substr(1) : '$' + f;
 }
 
+function isJsLibraryConfigIdentifier(ident) {
+  return ident.endsWith('__sig') || ident.endsWith('__proxy') || ident.endsWith('__asm') || ident.endsWith('__inline')
+   || ident.endsWith('__deps') || ident.endsWith('__postset') || ident.endsWith('__docs') || ident.endsWith('__import');
+}
+
 // JSifier
 function JSify(data, functionsOnly) {
   var mainPass = !functionsOnly;
@@ -75,7 +80,7 @@ function JSify(data, functionsOnly) {
       assert(!SIDE_MODULE, 'Cannot have both INCLUDE_FULL_LIBRARY and SIDE_MODULE set.')
       libFuncsToInclude = (MAIN_MODULE || SIDE_MODULE) ? DEFAULT_LIBRARY_FUNCS_TO_INCLUDE.slice(0) : [];
       for (var key in LibraryManager.library) {
-        if (!key.match(/__(deps|postset|inline|asm|sig)$/)) {
+        if (!isJsLibraryConfigIdentifier(key)) {
           libFuncsToInclude.push(key);
         }
       }
@@ -138,7 +143,7 @@ function JSify(data, functionsOnly) {
       if (typeof ident == 'function') return ident();
 
       // don't process any special identifiers. These are looked up when processing the base name of the identifier.
-      if (ident.endsWith('__sig') || ident.endsWith('__proxy') || ident.endsWith('__asm') || ident.endsWith('__inline') || ident.endsWith('__deps') || ident.endsWith('__postset')) {
+      if (isJsLibraryConfigIdentifier(ident)) {
         return '';
       }
 
@@ -162,16 +167,25 @@ function JSify(data, functionsOnly) {
         return;
       } else if ((!LibraryManager.library.hasOwnProperty(ident) && !LibraryManager.library.hasOwnProperty(ident + '__inline')) || SIDE_MODULE) {
         if (!(finalName in IMPLEMENTED_FUNCTIONS) && !LINKABLE) {
+          var msg = 'undefined symbol: ' + ident;
           if (ERROR_ON_UNDEFINED_SYMBOLS) {
-            error('undefined symbol: ' + ident);
+            error(msg);
+            if (WASM_BACKEND && !LLD_REPORT_UNDEFINED) {
+              warnOnce('Link with `-s LLD_REPORT_UNDEFINED` to get more information on undefined symbols');
+            }
             warnOnce('To disable errors for undefined symbols use `-s ERROR_ON_UNDEFINED_SYMBOLS=0`')
+            warnOnce(finalName + ' may need to be added to EXPORTED_FUNCTIONS if it arrives from a system library')
           } else if (VERBOSE || WARN_ON_UNDEFINED_SYMBOLS) {
-            warn('undefined symbol: ' + ident);
+            warn(msg);
           }
         }
         if (!RELOCATABLE) {
           // emit a stub that will fail at runtime
           LibraryManager.library[ident] = new Function("err('missing function: " + ident + "'); abort(-1);");
+          // We have already warned/errored about this function, so for the purposes of Closure use, mute all type checks
+          // regarding this function, marking ot a variadic function that can take in anything and return anything.
+          // (not useful to warn/error multiple times)
+          LibraryManager.library[ident + '__docs'] = '/** @type {function(...*):?} */';
         } else {
           var isGlobalAccessor = ident.startsWith('g$');
           var realIdent = ident;
@@ -235,6 +249,16 @@ function JSify(data, functionsOnly) {
       } else if (typeof snippet === 'function') {
         isFunction = true;
         snippet = processLibraryFunction(snippet, ident, finalName);
+        // Functions that start with '$' should not be imported to asm.js/wasm module, since they are
+        // intended to be exclusive to JS code only.
+        if (ident[0] != '$') {
+          Functions.libraryFunctions[finalName] = 1;
+        }
+      }
+
+      // If a JS library item specifies xxx_import: true, then explicitly mark that symbol to be imported
+      // to asm.js/wasm module.
+      if (LibraryManager.library[ident + '__import']) {
         Functions.libraryFunctions[finalName] = 1;
       }
 
@@ -326,7 +350,13 @@ function JSify(data, functionsOnly) {
           }
         });
       }
-      return depsText + contentText;
+
+      var commentText = '';
+      if (LibraryManager.library[ident + '__docs']) {
+        commentText = LibraryManager.library[ident + '__docs'] + '\n';
+      }
+
+      return depsText + commentText + contentText;
     }
 
     itemsDict.functionStub.push(item);
