@@ -26,7 +26,55 @@ mergeInto(LibraryManager.library, {
     addAtInit('if (!Module["noFSInit"] && !FS.init.initialized) FS.init();');
     addAtMain('FS.ignorePermissions = false;');
     addAtExit('FS.quit();');
-    return 'FS.staticInit();' +
+    // We must statically create FS.FSNode here so that it is created in a manner
+    // that is visible to Closure compiler. That lets us use type annotations for
+    // Closure to the "this" pointer in various node creation functions.
+    return `var FSNode = /** @constructor */ function(parent, name, mode, rdev) {
+  if (!parent) {
+    parent = this;  // root node sets parent to itself
+  }
+  this.parent = parent;
+  this.mount = parent.mount;
+  this.mounted = null;
+  this.id = FS.nextInode++;
+  this.name = name;
+  this.mode = mode;
+  this.node_ops = {};
+  this.stream_ops = {};
+  this.rdev = rdev;
+};
+var readMode = 292/*{{{ cDefine("S_IRUGO") }}}*/ | 73/*{{{ cDefine("S_IXUGO") }}}*/;
+var writeMode = 146/*{{{ cDefine("S_IWUGO") }}}*/;
+Object.defineProperties(FSNode.prototype, {
+ read: {
+  get: /** @this{FSNode} */function() {
+   return (this.mode & readMode) === readMode;
+  },
+  set: /** @this{FSNode} */function(val) {
+   val ? this.mode |= readMode : this.mode &= ~readMode;
+  }
+ },
+ write: {
+  get: /** @this{FSNode} */function() {
+   return (this.mode & writeMode) === writeMode;
+  },
+  set: /** @this{FSNode} */function(val) {
+   val ? this.mode |= writeMode : this.mode &= ~writeMode;
+  }
+ },
+ isFolder: {
+  get: /** @this{FSNode} */function() {
+   return FS.isDir(this.mode);
+  }
+ },
+ isDevice: {
+  get: /** @this{FSNode} */function() {
+   return FS.isChrdev(this.mode);
+  }
+ }
+});
+FS.FSNode = FSNode;
+FS.staticInit();` +
 #if USE_CLOSURE_COMPILER
            // Declare variable for Closure, FS.createPreloadedFile() below calls Browser.init()
            '/**@suppress {duplicate, undefinedVars}*/var Browser;' +
@@ -204,48 +252,6 @@ mergeInto(LibraryManager.library, {
       return FS.lookup(parent, name);
     },
     createNode: function(parent, name, mode, rdev) {
-      if (!FS.FSNode) {
-        FS.FSNode = function(parent, name, mode, rdev) {
-          if (!parent) {
-            parent = this;  // root node sets parent to itself
-          }
-          this.parent = parent;
-          this.mount = parent.mount;
-          this.mounted = null;
-          this.id = FS.nextInode++;
-          this.name = name;
-          this.mode = mode;
-          this.node_ops = {};
-          this.stream_ops = {};
-          this.rdev = rdev;
-        };
-
-        FS.FSNode.prototype = {};
-
-        // compatibility
-        var readMode = {{{ cDefine('S_IRUGO') }}} | {{{ cDefine('S_IXUGO') }}};
-        var writeMode = {{{ cDefine('S_IWUGO') }}};
-
-        // NOTE we must use Object.defineProperties instead of individual calls to
-        // Object.defineProperty in order to make closure compiler happy
-        Object.defineProperties(FS.FSNode.prototype, {
-          read: {
-            get: function() { return (this.mode & readMode) === readMode; },
-            set: function(val) { val ? this.mode |= readMode : this.mode &= ~readMode; }
-          },
-          write: {
-            get: function() { return (this.mode & writeMode) === writeMode; },
-            set: function(val) { val ? this.mode |= writeMode : this.mode &= ~writeMode; }
-          },
-          isFolder: {
-            get: function() { return FS.isDir(this.mode); }
-          },
-          isDevice: {
-            get: function() { return FS.isChrdev(this.mode); }
-          }
-        });
-      }
-
       var node = new FS.FSNode(parent, name, mode, rdev);
 
       FS.hashAddNode(node);
@@ -409,10 +415,8 @@ mergeInto(LibraryManager.library, {
     // SOCKFS is completed.
     createStream: function(stream, fd_start, fd_end) {
       if (!FS.FSStream) {
-        FS.FSStream = function(){};
-        FS.FSStream.prototype = {};
-        // compatibility
-        Object.defineProperties(FS.FSStream.prototype, {
+        FS.FSStream = /** @constructor */ function(){};
+        FS.FSStream.prototype = {
           object: {
             get: function() { return this.node; },
             set: function(val) { this.node = val; }
@@ -426,7 +430,7 @@ mergeInto(LibraryManager.library, {
           isAppend: {
             get: function() { return (this.flags & {{{ cDefine('O_APPEND') }}}); }
           }
-        });
+        };
       }
       // clone it, so we can return an instance of FSStream
       var newStream = new FS.FSStream();
@@ -1425,9 +1429,9 @@ mergeInto(LibraryManager.library, {
     },
     ensureErrnoError: function() {
       if (FS.ErrnoError) return;
-      FS.ErrnoError = function ErrnoError(errno, node) {
+      FS.ErrnoError = /** @this{Object} */ function ErrnoError(errno, node) {
         this.node = node;
-        this.setErrno = function(errno) {
+        this.setErrno = /** @this{Object} */ function(errno) {
           this.errno = errno;
 #if ASSERTIONS
           for (var key in ERRNO_CODES) {
@@ -1705,11 +1709,12 @@ mergeInto(LibraryManager.library, {
     // either --preload-file in emcc or FS.createPreloadedFile
     createLazyFile: function(parent, name, url, canRead, canWrite) {
       // Lazy chunked Uint8Array (implements get and length from Uint8Array). Actual getting is abstracted away for eventual reuse.
+      /** @constructor */
       function LazyUint8Array() {
         this.lengthKnown = false;
         this.chunks = []; // Loaded chunks. Index is the chunk number
       }
-      LazyUint8Array.prototype.get = function LazyUint8Array_get(idx) {
+      LazyUint8Array.prototype.get = /** @this{Object} */ function LazyUint8Array_get(idx) {
         if (idx > this.length-1 || idx < 0) {
           return undefined;
         }
@@ -1758,7 +1763,7 @@ mergeInto(LibraryManager.library, {
           xhr.send(null);
           if (!(xhr.status >= 200 && xhr.status < 300 || xhr.status === 304)) throw new Error("Couldn't load " + url + ". Status: " + xhr.status);
           if (xhr.response !== undefined) {
-            return new Uint8Array(xhr.response || []);
+            return new Uint8Array(/** @type{Array<number>} */(xhr.response || []));
           } else {
             return intArrayFromString(xhr.responseText || '', true);
           }
@@ -1792,7 +1797,7 @@ mergeInto(LibraryManager.library, {
         var lazyArray = new LazyUint8Array();
         Object.defineProperties(lazyArray, {
           length: {
-            get: function() {
+            get: /** @this{Object} */ function() {
               if(!this.lengthKnown) {
                 this.cacheLength();
               }
@@ -1800,7 +1805,7 @@ mergeInto(LibraryManager.library, {
             }
           },
           chunkSize: {
-            get: function() {
+            get: /** @this{Object} */ function() {
               if(!this.lengthKnown) {
                 this.cacheLength();
               }
@@ -1827,7 +1832,7 @@ mergeInto(LibraryManager.library, {
       // Add a function that defers querying the file size until it is asked the first time.
       Object.defineProperties(node, {
         usedBytes: {
-          get: function() { return this.contents.length; }
+          get: /** @this {FSNode} */ function() { return this.contents.length; }
         }
       });
       // override each stream op with one that tries to force load the lazy file first
