@@ -35,10 +35,8 @@
  */
 
 var LibraryGLFW = {
-  $GLFW__deps: ['emscripten_get_now', '$GL', '$Browser'],
-  $GLFW: {
-
-    Window: function(id, width, height, title, monitor, share) {
+  $GLFW_Window__docs: '/** @constructor */',
+  $GLFW_Window: function(id, width, height, title, monitor, share) {
       this.id = id;
       this.x = 0;
       this.y = 0;
@@ -80,6 +78,12 @@ var LibraryGLFW = {
       this.userptr = null;
     },
 
+  $GLFW__deps: ['emscripten_get_now', '$GL', '$Browser', '$GLFW_Window',
+#if FILESYSTEM
+    , '$FS'
+#endif
+  ],
+  $GLFW: {
     WindowFromId: function(id) {
       if (id <= 0 || !GLFW.windows) return null;
       return GLFW.windows[id - 1];
@@ -187,7 +191,7 @@ var LibraryGLFW = {
         case 0xDC:return 92; // DOM_VK_BACKSLASH -> GLFW_KEY_BACKSLASH
         case 0xDD:return 93; // DOM_VK_CLOSE_BRACKET -> GLFW_KEY_RIGHT_BRACKET
         case 0xC0:return 94; // DOM_VK_BACK_QUOTE -> GLFW_KEY_GRAVE_ACCENT
-        
+
 #if USE_GLFW == 2
         //#define GLFW_KEY_SPECIAL      256
         case 0x1B:return (256+1); // DOM_VK_ESCAPE -> GLFW_KEY_ESC
@@ -356,11 +360,11 @@ var LibraryGLFW = {
       if (charCode == 0 || (charCode >= 0x00 && charCode <= 0x1F)) return;
 
 #if USE_GLFW == 2
-      Module['dynCall_vii'](GLFW.active.charFunc, charCode, 1);
+      {{{ makeDynCall('vii') }}}(GLFW.active.charFunc, charCode, 1);
 #endif
 
 #if USE_GLFW == 3
-      Module['dynCall_vii'](GLFW.active.charFunc, GLFW.active.id, charCode);
+      {{{ makeDynCall('vii') }}}(GLFW.active.charFunc, GLFW.active.id, charCode);
 #endif
     },
 
@@ -378,12 +382,12 @@ var LibraryGLFW = {
       if (!GLFW.active.keyFunc) return;
 
 #if USE_GLFW == 2
-      Module['dynCall_vii'](GLFW.active.keyFunc, key, status);
+      {{{ makeDynCall('vii') }}}(GLFW.active.keyFunc, key, status);
 #endif
 
 #if USE_GLFW == 3
       if (repeat) status = 2; // GLFW_REPEAT
-      Module['dynCall_viiiii'](GLFW.active.keyFunc, GLFW.active.id, key, keyCode, status, GLFW.getModBits(GLFW.active));
+      {{{ makeDynCall('viiiii') }}}(GLFW.active.keyFunc, GLFW.active.id, key, keyCode, status, GLFW.getModBits(GLFW.active));
 #endif
     },
 
@@ -428,11 +432,11 @@ var LibraryGLFW = {
       if (event.target != Module["canvas"] || !GLFW.active.cursorPosFunc) return;
 
 #if USE_GLFW == 2
-      Module['dynCall_vii'](GLFW.active.cursorPosFunc, Browser.mouseX, Browser.mouseY);
+      {{{ makeDynCall('vii') }}}(GLFW.active.cursorPosFunc, Browser.mouseX, Browser.mouseY);
 #endif
 
 #if USE_GLFW == 3
-      Module['dynCall_vidd'](GLFW.active.cursorPosFunc, GLFW.active.id, Browser.mouseX, Browser.mouseY);
+      {{{ makeDynCall('vidd') }}}(GLFW.active.cursorPosFunc, GLFW.active.id, Browser.mouseX, Browser.mouseY);
 #endif
     },
 
@@ -456,7 +460,7 @@ var LibraryGLFW = {
       if (event.target != Module["canvas"] || !GLFW.active.cursorEnterFunc) return;
 
 #if USE_GLFW == 3
-      Module['dynCall_vii'](GLFW.active.cursorEnterFunc, GLFW.active.id, 1);
+      {{{ makeDynCall('vii') }}}(GLFW.active.cursorEnterFunc, GLFW.active.id, 1);
 #endif
     },
 
@@ -466,7 +470,7 @@ var LibraryGLFW = {
       if (event.target != Module["canvas"] || !GLFW.active.cursorEnterFunc) return;
 
 #if USE_GLFW == 3
-      Module['dynCall_vii'](GLFW.active.cursorEnterFunc, GLFW.active.id, 0);
+      {{{ makeDynCall('vii') }}}(GLFW.active.cursorEnterFunc, GLFW.active.id, 0);
 #endif
     },
 
@@ -491,11 +495,11 @@ var LibraryGLFW = {
       if (!GLFW.active.mouseButtonFunc) return;
 
 #if USE_GLFW == 2
-      Module['dynCall_vii'](GLFW.active.mouseButtonFunc, eventButton, status);
+      {{{ makeDynCall('vii') }}}(GLFW.active.mouseButtonFunc, eventButton, status);
 #endif
 
 #if USE_GLFW == 3
-      Module['dynCall_viiii'](GLFW.active.mouseButtonFunc, GLFW.active.id, eventButton, status, GLFW.getModBits(GLFW.active));
+      {{{ makeDynCall('viiii') }}}(GLFW.active.mouseButtonFunc, GLFW.active.id, eventButton, status, GLFW.getModBits(GLFW.active));
 #endif
     },
 
@@ -518,7 +522,7 @@ var LibraryGLFW = {
       if (!GLFW.active || !GLFW.active.scrollFunc || event.target != Module['canvas']) return;
 
 #if USE_GLFW == 2
-      Module['dynCall_vi'](GLFW.active.scrollFunc, GLFW.wheelPos);
+      {{{ makeDynCall('vi') }}}(GLFW.active.scrollFunc, GLFW.wheelPos);
 #endif
 
 #if USE_GLFW == 3
@@ -532,7 +536,7 @@ var LibraryGLFW = {
         sy = event.deltaY;
       }
 
-      Module['dynCall_vidd'](GLFW.active.scrollFunc, GLFW.active.id, sx, sy);
+      {{{ makeDynCall('vidd') }}}(GLFW.active.scrollFunc, GLFW.active.id, sx, sy);
 #endif
 
       event.preventDefault();
@@ -587,11 +591,11 @@ var LibraryGLFW = {
       if (!GLFW.active.windowSizeFunc) return;
 
 #if USE_GLFW == 2
-      Module['dynCall_vii'](GLFW.active.windowSizeFunc, GLFW.active.width, GLFW.active.height);
+      {{{ makeDynCall('vii') }}}(GLFW.active.windowSizeFunc, GLFW.active.width, GLFW.active.height);
 #endif
 
 #if USE_GLFW == 3
-      Module['dynCall_viii'](GLFW.active.windowSizeFunc, GLFW.active.id, GLFW.active.width, GLFW.active.height);
+      {{{ makeDynCall('viii') }}}(GLFW.active.windowSizeFunc, GLFW.active.id, GLFW.active.width, GLFW.active.height);
 #endif
     },
 
@@ -601,43 +605,10 @@ var LibraryGLFW = {
       if (!GLFW.active.framebufferSizeFunc) return;
 
 #if USE_GLFW == 3
-      Module['dynCall_viii'](GLFW.active.framebufferSizeFunc, GLFW.active.id, GLFW.active.width, GLFW.active.height);
+      {{{ makeDynCall('viii') }}}(GLFW.active.framebufferSizeFunc, GLFW.active.id, GLFW.active.width, GLFW.active.height);
 #endif
     },
 
-    requestFullscreen: function() {
-      var RFS = Module["canvas"]['requestFullscreen'] ||
-                Module["canvas"]['mozRequestFullScreen'] ||
-                Module["canvas"]['webkitRequestFullScreen'] ||
-                (function() {});
-      RFS.apply(Module["canvas"], []);
-    },
-
-    requestFullScreen: function() {
-      err('GLFW.requestFullScreen() is deprecated. Please call GLFW.requestFullscreen instead.');
-      GLFW.requestFullScreen = function() {
-        return GLFW.requestFullscreen();
-      }
-      return GLFW.requestFullscreen();
-    },
-
-    exitFullscreen: function() {
-      var CFS = document['exitFullscreen'] ||
-                document['cancelFullScreen'] ||
-                document['mozCancelFullScreen'] ||
-                document['webkitCancelFullScreen'] ||
-          (function() {});
-      CFS.apply(document, []);
-    },
-
-    cancelFullScreen: function() {
-      err('GLFW.cancelFullScreen() is deprecated. Please call GLFW.exitFullscreen instead.');
-      GLFW.cancelFullScreen = function() {
-        return GLFW.exitFullscreen();
-      }
-      return GLFW.exitFullscreen();
-    },
-
     getTime: function() {
       return _emscripten_get_now() / 1000;
     },
@@ -648,7 +619,7 @@ var LibraryGLFW = {
       var win = GLFW.WindowFromId(winid);
       if (!win) return;
 
-      win.title = Pointer_stringify(title);
+      win.title = UTF8ToString(title);
       if (GLFW.active.id == win.id) {
         document.title = win.title;
       }
@@ -684,7 +655,7 @@ var LibraryGLFW = {
               };
 
               if (GLFW.joystickFunc) {
-                Module['dynCall_vii'](GLFW.joystickFunc, joy, 0x00040001); // GLFW_CONNECTED
+                {{{ makeDynCall('vii') }}}(GLFW.joystickFunc, joy, 0x00040001); // GLFW_CONNECTED
               }
             }
 
@@ -702,7 +673,7 @@ var LibraryGLFW = {
               console.log('glfw joystick disconnected',joy);
 
               if (GLFW.joystickFunc) {
-                Module['dynCall_vii'](GLFW.joystickFunc, joy, 0x00040002); // GLFW_DISCONNECTED
+                {{{ makeDynCall('vii') }}}(GLFW.joystickFunc, joy, 0x00040002); // GLFW_DISCONNECTED
               }
 
               _free(GLFW.joys[joy].id);
@@ -718,38 +689,50 @@ var LibraryGLFW = {
 
     setKeyCallback: function(winid, cbfun) {
       var win = GLFW.WindowFromId(winid);
-      if (!win) return;
+      if (!win) return null;
+      var prevcbfun = win.keyFunc;
       win.keyFunc = cbfun;
+      return prevcbfun;
     },
 
     setCharCallback: function(winid, cbfun) {
       var win = GLFW.WindowFromId(winid);
-      if (!win) return;
+      if (!win) return null;
+      var prevcbfun = win.charFunc;
       win.charFunc = cbfun;
+      return prevcbfun;
     },
 
     setMouseButtonCallback: function(winid, cbfun) {
       var win = GLFW.WindowFromId(winid);
-      if (!win) return;
+      if (!win) return null;
+      var prevcbfun = win.mouseButtonFunc;
       win.mouseButtonFunc = cbfun;
+      return prevcbfun;
     },
 
     setCursorPosCallback: function(winid, cbfun) {
       var win = GLFW.WindowFromId(winid);
-      if (!win) return;
+      if (!win) return null;
+      var prevcbfun = win.cursorPosFunc;
       win.cursorPosFunc = cbfun;
+      return prevcbfun;
     },
 
     setScrollCallback: function(winid, cbfun) {
       var win = GLFW.WindowFromId(winid);
-      if (!win) return;
+      if (!win) return null;
+      var prevcbfun = win.scrollFunc;
       win.scrollFunc = cbfun;
+      return prevcbfun;
     },
 
     setDropCallback: function(winid, cbfun) {
       var win = GLFW.WindowFromId(winid);
-      if (!win) return;
+      if (!win) return null;
+      var prevcbfun = win.dropFunc;
       win.dropFunc = cbfun;
+      return prevcbfun;
     },
 
     onDrop: function(event) {
@@ -758,6 +741,7 @@ var LibraryGLFW = {
 
       event.preventDefault();
 
+#if FILESYSTEM
       var filenames = allocate(new Array(event.dataTransfer.files.length*4), 'i8*', ALLOC_NORMAL);
       var filenamesArray = [];
       var count = event.dataTransfer.files.length;
@@ -780,7 +764,7 @@ var LibraryGLFW = {
           var data = e.target.result;
           FS.writeFile(path, new Uint8Array(data));
           if (++written === count) {
-            Module['dynCall_viii'](GLFW.active.dropFunc, GLFW.active.id, count, filenames);
+            {{{ makeDynCall('viii') }}}(GLFW.active.dropFunc, GLFW.active.id, count, filenames);
 
             for (var i = 0; i < filenamesArray.length; ++i) {
               _free(filenamesArray[i]);
@@ -798,6 +782,7 @@ var LibraryGLFW = {
       for (var i = 0; i < count; ++i) {
         save(event.dataTransfer.files[i]);
       }
+#endif // FILESYSTEM
 
       return false;
     },
@@ -811,29 +796,36 @@ var LibraryGLFW = {
 
     setWindowSizeCallback: function(winid, cbfun) {
       var win = GLFW.WindowFromId(winid);
-      if (!win) return;
+      if (!win) return null;
+      var prevcbfun = win.windowSizeFunc;
       win.windowSizeFunc = cbfun;
-     
+
 #if USE_GLFW == 2
       // As documented in GLFW2 API (http://www.glfw.org/GLFWReference27.pdf#page=22), when size
       // callback function is set, it will be called with the current window size before this
       // function returns.
       // GLFW3 on the over hand doesn't have this behavior (https://github.com/glfw/glfw/issues/62).
-      if (!win.windowSizeFunc) return;
-      Module['dynCall_vii'](win.windowSizeFunc, win.width, win.height);
+      if (!win.windowSizeFunc) return null;
+      {{{ makeDynCall('vii') }}}(win.windowSizeFunc, win.width, win.height);
 #endif
+
+      return prevcbfun;
     },
 
     setWindowCloseCallback: function(winid, cbfun) {
       var win = GLFW.WindowFromId(winid);
-      if (!win) return;
+      if (!win) return null;
+      var prevcbfun = win.windowCloseFunc;
       win.windowCloseFunc = cbfun;
+      return prevcbfun;
     },
 
     setWindowRefreshCallback: function(winid, cbfun) {
       var win = GLFW.WindowFromId(winid);
-      if (!win) return;
+      if (!win) return null;
+      var prevcbfun = win.windowRefreshFunc;
       win.windowRefreshFunc = cbfun;
+      return prevcbfun;
     },
 
     onClickRequestPointerLock: function(e) {
@@ -954,9 +946,9 @@ var LibraryGLFW = {
 
       if (GLFW.active.id == win.id) {
         if (width == screen.width && height == screen.height) {
-          GLFW.requestFullscreen();
+          Browser.requestFullscreen();
         } else {
-          GLFW.exitFullscreen();
+          Browser.exitFullscreen();
           Browser.setCanvasSize(width, height);
           win.width = width;
           win.height = height;
@@ -966,17 +958,19 @@ var LibraryGLFW = {
       if (!win.windowSizeFunc) return;
 
 #if USE_GLFW == 2
-      Module['dynCall_vii'](win.windowSizeFunc, width, height);
+      {{{ makeDynCall('vii') }}}(win.windowSizeFunc, width, height);
 #endif
 
 #if USE_GLFW == 3
-      Module['dynCall_viii'](win.windowSizeFunc, win.id, width, height);
+      {{{ makeDynCall('viii') }}}(win.windowSizeFunc, win.id, width, height);
 #endif
     },
 
     createWindow: function(width, height, title, monitor, share) {
       var i, id;
-      for (i = 0; i < GLFW.windows.length && GLFW.windows[i] !== null; i++);
+      for (i = 0; i < GLFW.windows.length && GLFW.windows[i] !== null; i++) {
+        // no-op
+      }
       if (i > 0) throw "glfwCreateWindow only supports one window at time currently";
 
       // id for window
@@ -986,19 +980,21 @@ var LibraryGLFW = {
       if (width <= 0 || height <= 0) return 0;
 
       if (monitor) {
-        GLFW.requestFullscreen();
+        Browser.requestFullscreen();
       } else {
         Browser.setCanvasSize(width, height);
       }
 
       // Create context when there are no existing alive windows
-      for (i = 0; i < GLFW.windows.length && GLFW.windows[i] == null; i++);
+      for (i = 0; i < GLFW.windows.length && GLFW.windows[i] == null; i++) {
+        // no-op
+      }
       if (i == GLFW.windows.length) {
         var contextAttributes = {
           antialias: (GLFW.hints[0x0002100D] > 1), // GLFW_SAMPLES
           depth: (GLFW.hints[0x00021005] > 0),     // GLFW_DEPTH_BITS
           stencil: (GLFW.hints[0x00021006] > 0),   // GLFW_STENCIL_BITS
-          alpha: (GLFW.hints[0x00021004] > 0)      // GLFW_ALPHA_BITS 
+          alpha: (GLFW.hints[0x00021004] > 0)      // GLFW_ALPHA_BITS
         }
 #if OFFSCREEN_FRAMEBUFFER
         // TODO: Make GLFW explicitly aware of whether it is being proxied or not, and set these to true only when proxying is being performed.
@@ -1011,7 +1007,7 @@ var LibraryGLFW = {
       if (!Module.ctx) return 0;
 
       // Get non alive id
-      var win = new GLFW.Window(id, width, height, title, monitor, share);
+      var win = new GLFW_Window(id, width, height, title, monitor, share);
 
       // Set window to array
       if (id - 1 == GLFW.windows.length) {
@@ -1030,7 +1026,7 @@ var LibraryGLFW = {
 
 #if USE_GLFW == 3
       if (win.windowCloseFunc)
-        Module['dynCall_vi'](win.windowCloseFunc, win.id);
+        {{{ makeDynCall('vi') }}}(win.windowCloseFunc, win.id);
 #endif
 
       GLFW.windows[win.id - 1] = null;
@@ -1102,6 +1098,10 @@ var LibraryGLFW = {
     window.addEventListener("keypress", GLFW.onKeyPress, true);
     window.addEventListener("keyup", GLFW.onKeyup, true);
     window.addEventListener("blur", GLFW.onBlur, true);
+    Module["canvas"].addEventListener("touchmove", GLFW.onMousemove, true);
+    Module["canvas"].addEventListener("touchstart", GLFW.onMouseButtonDown, true);
+    Module["canvas"].addEventListener("touchcancel", GLFW.onMouseButtonUp, true);
+    Module["canvas"].addEventListener("touchend", GLFW.onMouseButtonUp, true);
     Module["canvas"].addEventListener("mousemove", GLFW.onMousemove, true);
     Module["canvas"].addEventListener("mousedown", GLFW.onMouseButtonDown, true);
     Module["canvas"].addEventListener("mouseup", GLFW.onMouseButtonUp, true);
@@ -1125,6 +1125,10 @@ var LibraryGLFW = {
     window.removeEventListener("keypress", GLFW.onKeyPress, true);
     window.removeEventListener("keyup", GLFW.onKeyup, true);
     window.removeEventListener("blur", GLFW.onBlur, true);
+    Module["canvas"].removeEventListener("touchmove", GLFW.onMousemove, true);
+    Module["canvas"].removeEventListener("touchstart", GLFW.onMouseButtonDown, true);
+    Module["canvas"].removeEventListener("touchcancel", GLFW.onMouseButtonUp, true);
+    Module["canvas"].removeEventListener("touchend", GLFW.onMouseButtonUp, true);
     Module["canvas"].removeEventListener("mousemove", GLFW.onMousemove, true);
     Module["canvas"].removeEventListener("mousedown", GLFW.onMouseButtonDown, true);
     Module["canvas"].removeEventListener("mouseup", GLFW.onMouseButtonUp, true);
@@ -1167,9 +1171,10 @@ var LibraryGLFW = {
     GLFW.initialTime = GLFW.getTime() - time;
   },
 
+  glfwExtensionSupported__deps: ['glGetString'],
   glfwExtensionSupported: function(extension) {
     if (!GLFW.extensions) {
-      GLFW.extensions = Pointer_stringify(_glGetString(0x1F03)).split(' ');
+      GLFW.extensions = UTF8ToString(_glGetString(0x1F03)).split(' ');
     }
 
     if (GLFW.extensions.indexOf(extension) != -1) return 1;
@@ -1200,7 +1205,9 @@ var LibraryGLFW = {
   },
 
   glfwSetErrorCallback: function(cbfun) {
+    var prevcbfun = GLFW.errorFunc;
     GLFW.errorFunc = cbfun;
+    return prevcbfun;
   },
 
   glfwWaitEventsTimeout: function(timeout) {},
@@ -1241,7 +1248,9 @@ var LibraryGLFW = {
   },
 
   glfwSetMonitorCallback: function(cbfun) {
+    var prevcbfun = GLFW.monitorFunc;
     GLFW.monitorFunc = cbfun;
+    return prevcbfun;
   },
 
   // TODO: implement
@@ -1327,11 +1336,15 @@ var LibraryGLFW = {
   },
 
   glfwIconifyWindow: function(winid) {
-    GLFW.iconifyWindow(winid);
+#if ASSERTIONS
+    warnOnce('glfwIconifyWindow is not implemented');
+#endif
   },
 
   glfwRestoreWindow: function(winid) {
-    GLFW.restoreWindow(winid);
+#if ASSERTIONS
+    warnOnce('glfwRestoreWindow is not implemented');
+#endif
   },
 
   glfwShowWindow: function(winid) {},
@@ -1364,32 +1377,38 @@ var LibraryGLFW = {
 
   glfwSetWindowPosCallback: function(winid, cbfun) {
     var win = GLFW.WindowFromId(winid);
-    if (!win) return;
+    if (!win) return null;
+    var prevcbfun = win.windowPosFunc;
     win.windowPosFunc = cbfun;
+    return prevcbfun;
   },
 
   glfwSetWindowSizeCallback: function(winid, cbfun) {
-    GLFW.setWindowSizeCallback(winid, cbfun);
+    return GLFW.setWindowSizeCallback(winid, cbfun);
   },
 
   glfwSetWindowCloseCallback: function(winid, cbfun) {
-    GLFW.setWindowCloseCallback(winid, cbfun);
+    return GLFW.setWindowCloseCallback(winid, cbfun);
   },
 
   glfwSetWindowRefreshCallback: function(winid, cbfun) {
-    GLFW.setWindowRefreshCallback(winid, cbfun);
+    return GLFW.setWindowRefreshCallback(winid, cbfun);
   },
 
   glfwSetWindowFocusCallback: function(winid, cbfun) {
     var win = GLFW.WindowFromId(winid);
-    if (!win) return;
+    if (!win) return null;
+    var prevcbfun = win.windowFocusFunc;
     win.windowFocusFunc = cbfun;
+    return prevcbfun;
   },
 
   glfwSetWindowIconifyCallback: function(winid, cbfun) {
     var win = GLFW.WindowFromId(winid);
-    if (!win) return;
+    if (!win) return null;
+    var prevcbfun = win.windowIconifyFunc;
     win.windowIconifyFunc = cbfun;
+    return prevcbfun;
   },
 
   glfwSetWindowIcon: function(winid, count, images) {},
@@ -1416,8 +1435,10 @@ var LibraryGLFW = {
 
   glfwSetFramebufferSizeCallback: function(winid, cbfun) {
     var win = GLFW.WindowFromId(winid);
-    if (!win) return;
+    if (!win) return null;
+    var prevcbfun = win.framebufferSizeFunc;
     win.framebufferSizeFunc = cbfun;
+    return prevcbfun;
   },
 
   glfwGetInputMode: function(winid, mode) {
@@ -1461,31 +1482,33 @@ var LibraryGLFW = {
   },
 
   glfwSetKeyCallback: function(winid, cbfun) {
-    GLFW.setKeyCallback(winid, cbfun);
+    return GLFW.setKeyCallback(winid, cbfun);
   },
 
   glfwSetCharCallback: function(winid, cbfun) {
-    GLFW.setCharCallback(winid, cbfun);
+    return GLFW.setCharCallback(winid, cbfun);
   },
 
   glfwSetCharModsCallback: function(winid, cbfun) { throw "glfwSetCharModsCallback not implemented."; },
 
   glfwSetMouseButtonCallback: function(winid, cbfun) {
-    GLFW.setMouseButtonCallback(winid, cbfun);
+    return GLFW.setMouseButtonCallback(winid, cbfun);
   },
 
   glfwSetCursorPosCallback: function(winid, cbfun) {
-    GLFW.setCursorPosCallback(winid, cbfun);
+    return GLFW.setCursorPosCallback(winid, cbfun);
   },
 
   glfwSetCursorEnterCallback: function(winid, cbfun) {
     var win = GLFW.WindowFromId(winid);
-    if (!win) return;
+    if (!win) return null;
+    var prevcbfun = win.cursorEnterFunc;
     win.cursorEnterFunc = cbfun;
+    return prevcbfun;
   },
 
   glfwSetScrollCallback: function(winid, cbfun) {
-    GLFW.setScrollCallback(winid, cbfun);
+    return GLFW.setScrollCallback(winid, cbfun);
   },
 
   glfwVulkanSupported: function() {
@@ -1493,7 +1516,7 @@ var LibraryGLFW = {
   },
 
   glfwSetDropCallback: function(winid, cbfun) {
-    GLFW.setDropCallback(winid, cbfun);
+    return GLFW.setDropCallback(winid, cbfun);
   },
 
   glfwGetTimerValue: function() { throw "glfwGetTimerValue is not implemented."; },
@@ -1610,11 +1633,15 @@ var LibraryGLFW = {
   },
 
   glfwIconifyWindow: function() {
-    GLFW.iconifyWindow(GLFW.active.id);
+#if ASSERTIONS
+    warnOnce('glfwIconifyWindow is not implemented');
+#endif
   },
 
   glfwRestoreWindow: function() {
-    GLFW.restoreWindow(GLFW.active.id);
+#if ASSERTIONS
+    warnOnce('glfwRestoreWindow is not implemented');
+#endif
   },
 
   glfwSwapBuffers: function() {
