@@ -7,7 +7,7 @@ mergeInto(LibraryManager.library, {
   $SOCKFS__postset: function() {
     addAtInit('SOCKFS.root = FS.mount(SOCKFS, {}, null);');
   },
-  $SOCKFS__deps: ['$FS'],
+  $SOCKFS__deps: ['$FS', '$ERRNO_CODES'], // TODO: avoid ERRNO_CODES
   $SOCKFS: {
     mount: function(mount) {
       // If Module['websocket'] has already been defined (e.g. for configuring
@@ -19,14 +19,14 @@ mergeInto(LibraryManager.library, {
       // object so we can register network callbacks from native JavaScript too.
       // For more documentation see system/include/emscripten/emscripten.h
       Module['websocket']._callbacks = {};
-      Module['websocket']['on'] = function(event, callback) {
+      Module['websocket']['on'] = /** @this{Object} */ function(event, callback) {
 	    if ('function' === typeof callback) {
 		  this._callbacks[event] = callback;
         }
 	    return this;
       };
 
-      Module['websocket'].emit = function(event, param) {
+      Module['websocket'].emit = /** @this{Object} */ function(event, param) {
 	    if ('function' === typeof this._callbacks[event]) {
 		  this._callbacks[event].call(this, param);
         }
@@ -194,12 +194,17 @@ mergeInto(LibraryManager.library, {
               }
             }
 
-            // The regex trims the string (removes spaces at the beginning and end, then splits the string by
-            // <any space>,<any space> into an Array. Whitespace removal is important for Websockify and ws.
-            subProtocols = subProtocols.replace(/^ +| +$/g,"").split(/ *, */);
+            // The default WebSocket options
+            var opts = undefined;
 
-            // The node ws library API for specifying optional subprotocol is slightly different than the browser's.
-            var opts = ENVIRONMENT_IS_NODE ? {'protocol': subProtocols.toString()} : subProtocols;
+            if (subProtocols !== 'null') {
+              // The regex trims the string (removes spaces at the beginning and end, then splits the string by
+              // <any space>,<any space> into an Array. Whitespace removal is important for Websockify and ws.
+              subProtocols = subProtocols.replace(/^ +| +$/g,"").split(/ *, */);
+
+              // The node ws library API for specifying optional subprotocol is slightly different than the browser's.
+              opts = ENVIRONMENT_IS_NODE ? {'protocol': subProtocols.toString()} : subProtocols;
+            }
 
             // some webservers (azure) does not support subprotocol header
             if (runtimeConfig && null === Module['websocket']['subprotocol']) {
@@ -214,14 +219,9 @@ mergeInto(LibraryManager.library, {
             var WebSocketConstructor;
 #if ENVIRONMENT_MAY_BE_NODE
             if (ENVIRONMENT_IS_NODE) {
-              WebSocketConstructor = require('ws');
+              WebSocketConstructor = /** @type{(typeof WebSocket)} */(require('ws'));
             } else
 #endif // ENVIRONMENT_MAY_BE_NODE
-#if ENVIRONMENT_MAY_BE_WEB
-            if (ENVIRONMENT_IS_WEB) {
-              WebSocketConstructor = window['WebSocket'];
-            } else
-#endif // ENVIRONMENT_MAY_BE_WEB
             {
               WebSocketConstructor = WebSocket;
             }
@@ -298,15 +298,20 @@ mergeInto(LibraryManager.library, {
         };
 
         function handleMessage(data) {
-          assert(typeof data !== 'string' && data.byteLength !== undefined);  // must receive an ArrayBuffer
-
-          // An empty ArrayBuffer will emit a pseudo disconnect event
-          // as recv/recvmsg will return zero which indicates that a socket
-          // has performed a shutdown although the connection has not been disconnected yet.
-          if (data.byteLength == 0) {
-            return;
+          if (typeof data === 'string') {
+            var encoder = new TextEncoder(); // should be utf-8
+            data = encoder.encode(data); // make a typed array from the string
+          } else {
+            assert(data.byteLength !== undefined); // must receive an ArrayBuffer
+            if (data.byteLength == 0) {
+              // An empty ArrayBuffer will emit a pseudo disconnect event
+              // as recv/recvmsg will return zero which indicates that a socket
+              // has performed a shutdown although the connection has not been disconnected yet.
+              return;
+            } else {
+              data = new Uint8Array(data); // make a typed array view on the array buffer
+            }
           }
-          data = new Uint8Array(data);  // make a typed array view on the array buffer
 
 #if SOCKET_DEBUG
           out('websocket handle message (' + data.byteLength + ' bytes): ' + [Array.prototype.slice.call(data)]);
@@ -746,7 +751,7 @@ mergeInto(LibraryManager.library, {
       }
     };
 
-    Module['noExitRuntime'] = true;
+    noExitRuntime = true;
     Module['websocket']['on'](event, callback ? _callback : null);
   },
   emscripten_set_socket_error_callback__deps: ['__set_network_callback'],
