@@ -100,7 +100,7 @@ var LibraryTracing = {
     },
 
     configureForGoogleWTF: function() {
-      if (window && window.wtf) {
+      if (window && window['wtf']) {
         EmscriptenTrace.googleWTFEnabled = true;
       } else {
         console.log('GOOGLE WTF NOT AVAILABLE TO ENABLE');
@@ -119,7 +119,7 @@ var LibraryTracing = {
     googleWTFEnterScope: function(name) {
       var scopeEvent = EmscriptenTrace.googleWTFData['cachedScopes'][name];
       if (!scopeEvent) {
-        scopeEvent = window.wtf.trace.events.createScope(name);
+        scopeEvent = window['wtf'].trace.events.createScope(name);
         EmscriptenTrace.googleWTFData['cachedScopes'][name] = scopeEvent;
       }
       var scope = scopeEvent();
@@ -128,7 +128,7 @@ var LibraryTracing = {
 
     googleWTFExitScope: function() {
       var scope = EmscriptenTrace.googleWTFData['scopeStack'].pop();
-      window.wtf.trace.leaveScope(scope);
+      window['wtf'].trace.leaveScope(scope);
     }
   },
 
@@ -137,8 +137,8 @@ var LibraryTracing = {
   },
 
   emscripten_trace_configure: function(collector_url, application) {
-    EmscriptenTrace.configure(Pointer_stringify(collector_url),
-                              Pointer_stringify(application));
+    EmscriptenTrace.configure(UTF8ToString(collector_url),
+                              UTF8ToString(application));
   },
 
   emscripten_trace_configure_for_test: function() {
@@ -154,7 +154,7 @@ var LibraryTracing = {
   },
 
   emscripten_trace_set_session_username: function(username) {
-    EmscriptenTrace.post(EmscriptenTrace.EVENT_USER_NAME, Pointer_stringify(username));
+    EmscriptenTrace.post(EmscriptenTrace.EVENT_USER_NAME, UTF8ToString(username));
   },
 
   emscripten_trace_record_frame_start: function() {
@@ -183,8 +183,8 @@ var LibraryTracing = {
     if (EmscriptenTrace.postEnabled) {
       var now = EmscriptenTrace.now();
       EmscriptenTrace.post([EmscriptenTrace.EVENT_LOG_MESSAGE, now,
-                            Pointer_stringify(channel),
-                            Pointer_stringify(message)]);
+                            UTF8ToString(channel),
+                            UTF8ToString(message)]);
     }
   },
 
@@ -195,7 +195,7 @@ var LibraryTracing = {
                             "MARK", message]);
     }
     if (EmscriptenTrace.googleWTFEnabled) {
-      window.wtf.trace.mark(message);
+      window['wtf'].trace.mark(message);
     }
   },
 
@@ -203,10 +203,10 @@ var LibraryTracing = {
     if (EmscriptenTrace.postEnabled) {
       var now = EmscriptenTrace.now();
       EmscriptenTrace.post([EmscriptenTrace.EVENT_LOG_MESSAGE, now,
-                            "MARK", Pointer_stringify(message)]);
+                            "MARK", UTF8ToString(message)]);
     }
     if (EmscriptenTrace.googleWTFEnabled) {
-      window.wtf.trace.mark(Pointer_stringify(message));
+      window['wtf'].trace.mark(UTF8ToString(message));
     }
   },
 
@@ -214,7 +214,7 @@ var LibraryTracing = {
     var now = EmscriptenTrace.now();
     var callstack = (new Error).stack;
     EmscriptenTrace.post([EmscriptenTrace.EVENT_REPORT_ERROR, now,
-                          Pointer_stringify(error), callstack]);
+                          UTF8ToString(error), callstack]);
   },
 
   emscripten_trace_record_allocation: function(address, size) {
@@ -247,7 +247,7 @@ var LibraryTracing = {
   emscripten_trace_annotate_address_type: function(address, type_name) {
     if (EmscriptenTrace.postEnabled) {
       EmscriptenTrace.post([EmscriptenTrace.EVENT_ANNOTATE_TYPE, address,
-                            Pointer_stringify(type_name)]);
+                            UTF8ToString(type_name)]);
     }
   },
 
@@ -262,13 +262,14 @@ var LibraryTracing = {
     if (EmscriptenTrace.postEnabled) {
       var memory_layout = {
         'static_base':  STATIC_BASE,
-        'static_top':   STATICTOP,
         'stack_base':   STACK_BASE,
         'stack_top':    STACKTOP,
         'stack_max':    STACK_MAX,
+#if !MINIMAL_RUNTIME
         'dynamic_base': DYNAMIC_BASE,
+#endif
         'dynamic_top':  HEAP32[DYNAMICTOP_PTR>>2],
-        'total_memory': TOTAL_MEMORY
+        'total_memory': HEAP8.length
       };
       var now = EmscriptenTrace.now();
       EmscriptenTrace.post([EmscriptenTrace.EVENT_MEMORY_LAYOUT,
@@ -314,10 +315,10 @@ var LibraryTracing = {
     if (EmscriptenTrace.postEnabled) {
       var now = EmscriptenTrace.now();
       EmscriptenTrace.post([EmscriptenTrace.EVENT_ENTER_CONTEXT,
-                            now, Pointer_stringify(name)]);
+                            now, UTF8ToString(name)]);
     }
     if (EmscriptenTrace.googleWTFEnabled) {
-      EmscriptenTrace.googleWTFEnterScope(Pointer_stringify(name));
+      EmscriptenTrace.googleWTFEnterScope(UTF8ToString(name));
     }
   },
 
@@ -335,15 +336,15 @@ var LibraryTracing = {
     if (EmscriptenTrace.postEnabled) {
       var now = EmscriptenTrace.now();
       EmscriptenTrace.post([EmscriptenTrace.EVENT_TASK_START,
-                            now, task_id, Pointer_stringify(name)]);
+                            now, task_id, UTF8ToString(name)]);
     }
   },
 
   emscripten_trace_task_associate_data: function(key, value) {
     if (EmscriptenTrace.postEnabled) {
       EmscriptenTrace.post([EmscriptenTrace.EVENT_TASK_ASSOCIATE_DATA,
-                            Pointer_stringify(key),
-                            Pointer_stringify(value)]);
+                            UTF8ToString(key),
+                            UTF8ToString(value)]);
     }
   },
 
@@ -351,7 +352,7 @@ var LibraryTracing = {
     if (EmscriptenTrace.postEnabled) {
       var now = EmscriptenTrace.now();
       EmscriptenTrace.post([EmscriptenTrace.EVENT_TASK_SUSPEND,
-                            now, Pointer_stringify(explanation)]);
+                            now, UTF8ToString(explanation)]);
     }
   },
 
@@ -359,7 +360,7 @@ var LibraryTracing = {
     if (EmscriptenTrace.postEnabled) {
       var now = EmscriptenTrace.now();
       EmscriptenTrace.post([EmscriptenTrace.EVENT_TASK_RESUME,
-                            now, task_id, Pointer_stringify(explanation)]);
+                            now, task_id, UTF8ToString(explanation)]);
     }
   },
 
