@@ -11,8 +11,13 @@ function assert(condition, text) {
 }
 #endif
 
+/** @param {string|number=} what */
 function abort(what) {
+#if ASSERTIONS
+  throw new Error(what);
+#else
   throw what;
+#endif
 }
 
 var tempRet0 = 0;
@@ -50,12 +55,16 @@ Module['wasm'] = base64Decode('{{{ getQuoted("WASM_BINARY_DATA") }}}');
 #include "runtime_sab_polyfill.js"
 
 #if USE_PTHREADS
+var STATIC_BASE = {{{ GLOBAL_BASE }}};
+
 if (!ENVIRONMENT_IS_PTHREAD) {
 #endif
 
 var GLOBAL_BASE = {{{ GLOBAL_BASE }}},
     TOTAL_STACK = {{{ TOTAL_STACK }}},
+#if !USE_PTHREADS
     STATIC_BASE = {{{ GLOBAL_BASE }}},
+#endif
     STACK_BASE = {{{ getQuoted('STACK_BASE') }}},
     STACKTOP = STACK_BASE,
     STACK_MAX = {{{ getQuoted('STACK_MAX') }}}
@@ -66,15 +75,15 @@ var GLOBAL_BASE = {{{ GLOBAL_BASE }}},
 
 #if WASM
 
-#if ALLOW_MEMORY_GROWTH && WASM_MEM_MAX != -1
-var wasmMaximumMemory = {{{ WASM_MEM_MAX }}};
+#if ALLOW_MEMORY_GROWTH && MAXIMUM_MEMORY != -1
+var wasmMaximumMemory = {{{ MAXIMUM_MEMORY }}};
 #else
-var wasmMaximumMemory = {{{ TOTAL_MEMORY }}};
+var wasmMaximumMemory = {{{ INITIAL_MEMORY }}};
 #endif
 
 var wasmMemory = new WebAssembly.Memory({
-  'initial': {{{ TOTAL_MEMORY }}} >> 16
-#if USE_PTHREADS || !ALLOW_MEMORY_GROWTH || WASM_MEM_MAX != -1
+  'initial': {{{ INITIAL_MEMORY }}} >> 16
+#if USE_PTHREADS || !ALLOW_MEMORY_GROWTH || MAXIMUM_MEMORY != -1
   , 'maximum': wasmMaximumMemory >> 16
 #endif
 #if USE_PTHREADS
@@ -82,6 +91,15 @@ var wasmMemory = new WebAssembly.Memory({
 #endif
   });
 
+var buffer = wasmMemory.buffer;
+
+#if USE_PTHREADS
+}
+#if ASSERTIONS
+assert(buffer instanceof SharedArrayBuffer, 'requested a shared WebAssembly.Memory but the returned buffer is not a SharedArrayBuffer, indicating that while the browser has SharedArrayBuffer it does not have WebAssembly threads support - you may need to set a flag');
+#endif
+#endif
+
 var wasmTable = new WebAssembly.Table({
   'initial': {{{ getQuoted('WASM_TABLE_SIZE') }}},
 #if !ALLOW_TABLE_GROWTH
@@ -94,18 +112,12 @@ var wasmTable = new WebAssembly.Table({
   'element': 'anyfunc'
 });
 
-var buffer = wasmMemory.buffer;
-
-#if USE_PTHREADS && ASSERTIONS
-assert(buffer instanceof SharedArrayBuffer, 'requested a shared WebAssembly.Memory but the returned buffer is not a SharedArrayBuffer, indicating that while the browser has SharedArrayBuffer it does not have WebAssembly threads support - you may need to set a flag');
-#endif
-
 #else
 
 #if USE_PTHREADS
-var buffer = new SharedArrayBuffer({{{ TOTAL_MEMORY }}});
+var buffer = new SharedArrayBuffer({{{ INITIAL_MEMORY }}});
 #else
-var buffer = new ArrayBuffer({{{ TOTAL_MEMORY }}});
+var buffer = new ArrayBuffer({{{ INITIAL_MEMORY }}});
 #endif
 
 #if USE_PTHREADS
@@ -116,14 +128,20 @@ var buffer = new ArrayBuffer({{{ TOTAL_MEMORY }}});
 
 #if ASSERTIONS
 var WASM_PAGE_SIZE = 65536;
-assert(STACK_BASE % 16 === 0, 'stack must start aligned');
-assert(({{{ getQuoted('DYNAMIC_BASE') }}}) % 16 === 0, 'heap must start aligned');
-assert({{{ TOTAL_MEMORY }}} >= TOTAL_STACK, 'TOTAL_MEMORY should be larger than TOTAL_STACK, was ' + {{{ TOTAL_MEMORY }}} + '! (TOTAL_STACK=' + TOTAL_STACK + ')');
-assert({{{ TOTAL_MEMORY }}} % WASM_PAGE_SIZE === 0);
-#if WASM_MEM_MAX != -1
-assert({{{ WASM_MEM_MAX }}} % WASM_PAGE_SIZE == 0);
+#if USE_PTHREADS
+if (!ENVIRONMENT_IS_PTHREAD) {
+#endif
+assert(STACK_BASE % 16 === 0, 'stack must start aligned to 16 bytes, STACK_BASE==' + STACK_BASE);
+assert(({{{ getQuoted('DYNAMIC_BASE') }}}) % 16 === 0, 'heap must start aligned to 16 bytes, DYNAMIC_BASE==' + {{{ getQuoted('DYNAMIC_BASE') }}});
+assert({{{ INITIAL_MEMORY }}} >= TOTAL_STACK, 'INITIAL_MEMORY should be larger than TOTAL_STACK, was ' + {{{ INITIAL_MEMORY }}} + '! (TOTAL_STACK=' + TOTAL_STACK + ')');
+assert({{{ INITIAL_MEMORY }}} % WASM_PAGE_SIZE === 0);
+#if MAXIMUM_MEMORY != -1
+assert({{{ MAXIMUM_MEMORY }}} % WASM_PAGE_SIZE == 0);
+#endif
+assert(buffer.byteLength === {{{ INITIAL_MEMORY }}});
+#if USE_PTHREADS
+}
 #endif
-assert(buffer.byteLength === {{{ TOTAL_MEMORY }}});
 #endif // ASSERTIONS
 
 #if ALLOW_MEMORY_GROWTH
@@ -156,11 +174,17 @@ var HEAPF32 = new Float32Array(buffer);
 var HEAPF64 = new Float64Array(buffer);
 #endif
 
+#if USE_PTHREADS && ((MEM_INIT_METHOD == 1 && !MEM_INIT_IN_WASM && !SINGLE_FILE) || (SINGLE_FILE && !WASM && !WASM_BACKEND) || USES_DYNAMIC_ALLOC)
+if (!ENVIRONMENT_IS_PTHREAD) {
+#endif
+
 #if MEM_INIT_METHOD == 1 && !MEM_INIT_IN_WASM && !SINGLE_FILE
+
 #if ASSERTIONS
 if (!Module['mem']) throw 'Must load memory initializer as an ArrayBuffer in to variable Module.mem before adding compiled output .js script to the DOM';
 #endif
 HEAPU8.set(new Uint8Array(Module['mem']), GLOBAL_BASE);
+
 #endif
 
 #if SINGLE_FILE && !WASM && !WASM_BACKEND
@@ -169,7 +193,11 @@ HEAPU8.set(base64Decode('{{{ getQuoted("BASE64_MEMORY_INITIALIZER") }}}'), GLOBA
 #endif
 
 #if USES_DYNAMIC_ALLOC
-HEAP32[DYNAMICTOP_PTR>>2] = {{{ getQuoted('DYNAMIC_BASE') }}};
+  HEAP32[DYNAMICTOP_PTR>>2] = {{{ getQuoted('DYNAMIC_BASE') }}};
+#endif
+
+#if USE_PTHREADS && ((MEM_INIT_METHOD == 1 && !MEM_INIT_IN_WASM && !SINGLE_FILE) || (SINGLE_FILE && !WASM && !WASM_BACKEND) || USES_DYNAMIC_ALLOC)
+}
 #endif
 
 #include "runtime_stack_check.js"
@@ -185,6 +213,31 @@ var wasmOffsetConverter;
 #include "wasm_offset_converter.js"
 #endif
 
+#if EXIT_RUNTIME
+
+function callRuntimeCallbacks(callbacks) {
+  while(callbacks.length > 0) {
+    var callback = callbacks.shift();
+    if (typeof callback == 'function') {
+      callback();
+      continue;
+    }
+    var func = callback.func;
+    if (typeof func === 'number') {
+      if (callback.arg === undefined) {
+        dynCall_v(func);
+      } else {
+        dynCall_vi(func, callback.arg);
+      }
+    } else {
+      func(callback.arg === undefined ? null : callback.arg);
+    }
+  }
+}
+
+var __ATEXIT__    = []; // functions called during shutdown
+#endif
+
 #if ASSERTIONS
 var runtimeInitialized = false;
 
@@ -192,15 +245,17 @@ var runtimeInitialized = false;
 var runtimeExited = false;
 #endif
 
+/** @param {number|boolean=} ignore */
 {{{ unSign }}}
+/** @param {number|boolean=} ignore */
 {{{ reSign }}}
 
 #include "runtime_math.js"
 
 var memoryInitializer = null;
 
-#if MEMORYPROFILER
 #include "memoryprofiler.js"
-#endif
+
+#include "runtime_debug.js"
 
 // === Body ===
