@@ -92,22 +92,33 @@ function convertJsFunctionToWasm(func, sig) {
 #endif // WASM2JS
 }
 
+var freeTableIndexes = [];
+
 // Add a wasm function to the table.
 function addFunctionWasm(func, sig) {
   var table = wasmTable;
-  var ret = table.length;
-
-  // Grow the table
-  try {
-    table.grow(1);
-  } catch (err) {
-    if (!(err instanceof RangeError)) {
-      throw err;
+  var ret;
+  // Reuse a free index if there is one, otherwise grow.
+  if (freeTableIndexes.length) {
+    ret = freeTableIndexes.pop();
+  } else {
+    ret = table.length;
+    // Grow the table
+    try {
+      table.grow(1);
+    } catch (err) {
+      if (!(err instanceof RangeError)) {
+        throw err;
+      }
+#if WASM_BACKEND
+      throw 'Unable to grow wasm table. Set ALLOW_TABLE_GROWTH.';
+#else
+      throw 'Unable to grow wasm table. Use a higher value for RESERVED_FUNCTION_POINTERS or set ALLOW_TABLE_GROWTH.';
+#endif
     }
-    throw 'Unable to grow wasm table. Use a higher value for RESERVED_FUNCTION_POINTERS or set ALLOW_TABLE_GROWTH.';
   }
 
-  // Insert new element
+  // Set the new value.
   try {
     // Attempting to call this with JS function will cause of table.set() to fail
     table.set(ret, func);
@@ -124,7 +135,7 @@ function addFunctionWasm(func, sig) {
 }
 
 function removeFunctionWasm(index) {
-  // TODO(sbc): Look into implementing this to allow re-using of table slots
+  freeTableIndexes.push(index);
 }
 #endif
 
@@ -162,8 +173,8 @@ function addFunction(func, sig) {
   alignFunctionTables(); // TODO: we should rely on this being an invariant
   var tables = getFunctionTables();
   var ret = -1;
-  for (var sig in tables) {
-    var table = tables[sig];
+  for (var signature in tables) {
+    var table = tables[signature];
     if (ret < 0) ret = table.length;
     else assert(ret === table.length);
     table.push(func);
