@@ -104,11 +104,6 @@ var MEM_INIT_METHOD = 0;
 // it will fail silently.
 var TOTAL_STACK = 5*1024*1024;
 
-// The total amount of memory to use. Using more memory than this will
-// cause us to expand the heap, which can be costly with typed arrays:
-// we need to copy the old heap into a new one in that case.
-var TOTAL_MEMORY = 16777216;
-
 // What malloc()/free() to use, out of
 //  * dlmalloc - a powerful general-purpose malloc
 //  * emmalloc - a simple and compact malloc designed for emscripten
@@ -126,7 +121,7 @@ var MALLOC = "dlmalloc";
 // but makes sense for the web since we have a fixed amount of memory that
 // must all be allocated up front, and so (a) failing mallocs are much more
 // likely than on other platforms, and (b) people need a way to find out
-// how big that initial allocation (TOTAL_MEMORY) must be.
+// how big that initial allocation (INITIAL_MEMORY) must be.
 // If you set this to 0, then you get the standard malloc behavior of
 // returning NULL (0) when it fails.
 var ABORTING_MALLOC = 1;
@@ -137,8 +132,28 @@ var ABORTING_MALLOC = 1;
 // [fastcomp-only]
 var FAST_UNROLLED_MEMCPY_AND_MEMSET = 1;
 
+// The initial amount of memory to use. Using more memory than this will
+// cause us to expand the heap, which can be costly with typed arrays:
+// we need to copy the old heap into a new one in that case.
+// If ALLOW_MEMORY_GROWTH is set, this initial amount of memory can increase
+// later; if not, then it is the final and total amount of memory.
+//
+// (This option was formerly called TOTAL_MEMORY.)
+var INITIAL_MEMORY = 16777216;
+
+// Set the maximum size of memory in the wasm module (in bytes). Without this,
+// INITIAL_MEMORY is used (as it is used for the initial value), or if memory
+// growth is enabled, the default value here (-1) is to have no limit, but you
+// can set this to set a maximum size that growth will stop at.
+//
+// This setting only matters for wasm, as in asm.js there is no place to set
+// a maximum, and only when ALLOW_MEMORY_GROWTH is set.
+//
+// (This option was formerly called WASM_MEM_MAX and BINARYEN_MEM_MAX.)
+var MAXIMUM_MEMORY = -1;
+
 // If false, we abort with an error if we try to allocate more memory than
-// we can (TOTAL_MEMORY). If true, we will grow the memory arrays at
+// we can (INITIAL_MEMORY). If true, we will grow the memory arrays at
 // runtime, seamlessly and dynamically. This has a performance cost in asm.js,
 // both during the actual growth and in general (the latter is because in
 // that case we must be careful about optimizations, in particular the
@@ -248,6 +263,12 @@ var SIMD = 0;
 // Whether closure compiling is being run on this output
 var USE_CLOSURE_COMPILER = 0;
 
+// Specifies how warnings emitted by Closure are treated. Possible
+// options: 'quiet', 'warn', 'error'. If set to 'warn', Closure warnings are printed
+// out to console. If set to 'error', Closure warnings are treated like errors,
+// similar to -Werror compiler flag.
+var CLOSURE_WARNINGS = 'quiet';
+
 // If set to 1, each asm.js/wasm module export is individually declared with a
 // JavaScript "var" definition. This is the simple and recommended approach.
 // However, this does increase code size (especially if you have many such
@@ -289,6 +310,7 @@ var SAFE_HEAP_LOG = 0;
 // In asm.js mode, we cannot simply add function pointers to function tables, so
 // we reserve some slots for them. An alternative to this is to use
 // EMULATED_FUNCTION_POINTERS, in which case we don't need to reserve.
+// [fastcomp-only]
 var RESERVED_FUNCTION_POINTERS = 0;
 
 // Whether to allow function pointers to alias if they have a different type.
@@ -479,7 +501,7 @@ var MAX_WEBGL_VERSION = 1;
 var WEBGL2_BACKWARDS_COMPATIBILITY_EMULATION = 0;
 
 // Forces support for all GLES3 features, not just the WebGL2-friendly subset.
-// This automatically turns out FULL_ES2.
+// This automatically turns on FULL_ES2 and WebGL2 support.
 var FULL_ES3 = 0;
 
 // Includes code to emulate various desktop GL features. Incomplete but useful
@@ -615,18 +637,14 @@ var NODEJS_CATCH_REJECTION = 1;
 // On upstream this uses the Asyncify pass in Binaryen. TODO: whitelist, coroutines
 var ASYNCIFY = 0;
 
-// The imports which can do a sync operation. If you add more you will need to
-// add them to here.
-// If the names here do not include a "module." prefix, the module is assumed
-// to be "env".
-var ASYNCIFY_IMPORTS = [
-  'emscripten_sleep', 'emscripten_wget', 'emscripten_wget_data', 'emscripten_idb_load',
-  'emscripten_idb_store', 'emscripten_idb_delete', 'emscripten_idb_exists',
-  'emscripten_idb_load_blob', 'emscripten_idb_store_blob', 'SDL_Delay',
-  'emscripten_scan_registers', 'emscripten_lazy_load_code',
-  'emscripten_fiber_swap',
-  'wasi_snapshot_preview1.fd_sync', '__wasi_fd_sync',
-];
+// Imports which can do a sync operation, in addition to the default ones that
+// emscripten defines like emscripten_sleep. If you add more you will need to
+// mention them to here, or else they will not work (in ASSERTIONS builds an
+// error will be shown).
+// Note that this list used to contain the default ones, which meant that you
+// had to list them when adding your own; the default ones are now added
+// automatically.
+var ASYNCIFY_IMPORTS = [];
 
 // Whether indirect calls can be on the stack during an unwind/rewind.
 // If you know they cannot, then setting this can be extremely helpful, as otherwise asyncify
@@ -684,8 +702,8 @@ var ASYNCIFY_DEBUG = 0;
 // included (either automatically from linking, or due to being in
 // DEFAULT_LIBRARY_FUNCS_TO_INCLUDE).
 // Note that the name may be slightly misleading, as this is for any JS library
-// element, and not just methods. For example, we export the Runtime object by
-// having "Runtime" in this list.
+// element, and not just methods. For example, we can export the FS object by
+// having "FS" in this list.
 var EXPORTED_RUNTIME_METHODS = [];
 
 // Additional methods to those in EXPORTED_RUNTIME_METHODS. Adjusting that list
@@ -716,7 +734,7 @@ var EXTRA_EXPORTED_RUNTIME_METHODS = [];
 // FIXME: should this just be  0  if we want everything?
 var INCOMING_MODULE_JS_API = [
   'ENVIRONMENT', 'GL_MAX_TEXTURE_IMAGE_UNITS', 'SDL_canPlayWithWebAudio',
-  'SDL_numSimultaneouslyQueuedBuffers', 'TOTAL_MEMORY', 'wasmMemory', 'arguments',
+  'SDL_numSimultaneouslyQueuedBuffers', 'INITIAL_MEMORY', 'wasmMemory', 'arguments',
   'buffer', 'canvas', 'doNotCaptureKeyboard', 'dynamicLibraries',
   'elementPointerLock', 'extraStackTrace', 'forcedAspectRatio',
   'instantiateWasm', 'keyboardListeningElementfreePreloadedMediaOnUse',
@@ -818,13 +836,7 @@ var RETAIN_COMPILER_SETTINGS = 0;
 // may be slightly misleading, as this is for any JS library element, and not
 // just functions. For example, you can include the Browser object by adding
 // "$Browser" to this list.
-var DEFAULT_LIBRARY_FUNCS_TO_INCLUDE = [
-  'memcpy',
-  'memset',
-  'malloc',
-  'free',
-  'emscripten_get_heap_size', // Used by dynamicAlloc() and -s FETCH=1
-];
+var DEFAULT_LIBRARY_FUNCS_TO_INCLUDE = [];
 
 // This list is also used to determine auto-exporting of library dependencies
 // (i.e., functions that might be dependencies of JS library functions, that if
@@ -914,7 +926,7 @@ var LINKABLE = 0;
 var STRICT = 0;
 
 // Automatically attempt to add archive indexes at link time to archives that 
-// don't already have them.  This can heppen when GNU ar or GNU ranlib is used
+// don't already have them.  This can happen when GNU ar or GNU ranlib is used
 // rather than `llvm-ar` or `emar` since the former don't understand the wasm
 // object format.
 // [link]
@@ -1211,13 +1223,6 @@ var STANDALONE_WASM = 0;
 // environment.
 var WASM_BACKEND = 0;
 
-// Whether to compile object files as wasm as opposed to the default
-// of using LLVM IR.
-// Setting to zero will enable LTO and at link time will also enable bitcode
-// versions of the standard libraries.
-// [compile+link]
-var WASM_OBJECT_FILES = 1;
-
 // An optional comma-separated list of script hooks to run after binaryen,
 // in binaryen's /scripts dir.
 var BINARYEN_SCRIPTS = "";
@@ -1262,14 +1267,6 @@ var BINARYEN_PASSES = "";
 // appended at the end of the list of passes.
 var BINARYEN_EXTRA_PASSES = "";
 
-// Set the maximum size of memory in the wasm module (in bytes). Without this,
-// TOTAL_MEMORY is used (as it is used for the initial value), or if memory
-// growth is enabled, the default value here (-1) is to have no limit, but you
-// can set this to set a maximum size that growth will stop at.
-//
-// (This option was formerly called BINARYEN_MEM_MAX)
-var WASM_MEM_MAX = -1;
-
 // Whether to compile the wasm asynchronously, which is more efficient and does
 // not block the main thread. This is currently required for all but the
 // smallest modules to run in chrome.
@@ -1379,15 +1376,40 @@ var IN_TEST_HARNESS = 0;
 // [compile+link] - affects user code at compile and system libraries at link
 var USE_PTHREADS = 0;
 
-// PTHREAD_POOL_SIZE specifies the number of web workers that are created
-// before the main runtime is initialized. If 0, workers are created on
-// demand. If PTHREAD_POOL_DELAY_LOAD = 0, then the workers will be fully
-// loaded (available for use) prior to the main runtime being initialized. If
-// PTHREAD_POOL_DELAY_LOAD = 1, then the workers will only be created and
-// have their runtimes loaded on demand after the main runtime is initialized.
-// Note that this means that the workers cannot be joined from the main thread
-// unless PROXY_TO_PTHREAD is used.
-var PTHREAD_POOL_SIZE = 0;
+// In web browsers, Workers cannot be created while the main browser thread
+// is executing JS/Wasm code, but the main thread must regularly yield back
+// to the browser event loop for Worker initialization to occur.
+// This means that pthread_create() is essentially an asynchronous operation
+// when called from the main browser thread, and the main thread must
+// repeatedly yield back to the JS event loop in order for the thread to
+// actually start.
+// If your application needs to be able to synchronously create new threads,
+// you can pre-create a pthread pool by specifying -s PTHREAD_POOL_SIZE=x,
+// in which case the specified number of Workers will be preloaded into a pool
+// before the application starts, and that many threads can then be available
+// for synchronous creation.
+// Note that this setting is a string, and will be emitted in the JS code
+// (directly, with no extra quotes) so that if you set it to '5' then 5 workers
+// will be used in the pool, and so forth. The benefit of this being a string
+// is that you can set it to something like
+// 'navigator.hardwareConcurrency' (which will use the number of cores the
+// browser reports, and is how you can get exactly enough workers for a
+// threadpool equal to the number of cores).
+// [link] - affects generated JS runtime code at link time
+var PTHREAD_POOL_SIZE = '';
+
+// If your application does not need the ability to synchronously create
+// threads, but it would still like to opportunistically speed up initial thread
+// startup time by prewarming a pool of Workers, you can specify the size of
+// the pool with -s PTHREAD_POOL_SIZE=x, but then also specify
+// -s PTHREAD_POOL_DELAY_LOAD=1, which will cause the runtime to not wait up at
+// startup for the Worker pool to finish loading. Instead, the runtime will
+// immediately start up and the Worker pool will asynchronously spin up in
+// parallel on the background. This can shorten the time that pthread_create()
+// calls take to actually start a thread, but without actually slowing down
+// main application startup speed. If PTHREAD_POOL_DELAY_LOAD=0 (default),
+// then the runtime will wait for the pool to start up before running main().
+// [link] - affects generated JS runtime code at link time
 var PTHREAD_POOL_DELAY_LOAD = 0;
 
 // If not explicitly specified, this is the stack size to use for newly created
@@ -1551,9 +1573,9 @@ var SINGLE_FILE = 0;
 // to execute the file without the accompanying JS file.
 var EMIT_EMSCRIPTEN_METADATA = 0;
 
-// If set to 1, all JS libraries will be automaticially available at link time.
+// If set to 1, all JS libraries will be automatically available at link time.
 // This gets set to 0 in STRICT mode (or with MINIMAL_RUNTIME) which mean you
-// need to explictly specify -lfoo.js in at link time in order to access
+// need to explicitly specify -lfoo.js in at link time in order to access
 // library function in library_foo.js.
 var AUTO_JS_LIBRARIES = 1;
 
@@ -1708,6 +1730,13 @@ var DISABLE_EXCEPTION_THROWING = 0;
 // across function boundaries.
 var USE_OFFSET_CONVERTER = 0;
 
+// If set to 1, the JS compiler is run before wasm-ld so that the linker can
+// report undefined symbols within the binary.  Without this option that linker
+// doesn't know which symmbols might be defined JS and so reporting of undefined
+// symbols is deleyed until the JS compiler is run.
+// [link]
+var LLD_REPORT_UNDEFINED = 0;
+
 //===========================================
 // Internal, used for testing only, from here
 //===========================================
@@ -1737,6 +1766,7 @@ var ASM_PRIMITIVE_VARS = ['__THREW__', 'threwValue', 'setjmpId', 'tempInt', 'tem
 // This allows existing build systems to keep specifying one of the supported
 // settings, for backwards compatibility.
 var LEGACY_SETTINGS = [
+  ['BINARYEN', 'WASM'],
   ['BINARYEN_ASYNC_COMPILATION', 'WASM_ASYNC_COMPILATION'],
   ['UNALIGNED_MEMORY', [0], 'forced unaligned memory not supported in fastcomp'],
   ['FORCE_ALIGNED_MEMORY', [0], 'forced aligned memory is not supported in fastcomp'],
@@ -1753,5 +1783,10 @@ var LEGACY_SETTINGS = [
   ['EMITTING_JS', [1], 'The new STANDALONE_WASM flag replaces this (replace EMITTING_JS=0 with STANDALONE_WASM=1)'],
   ['SKIP_STACK_IN_SMALL', [0, 1], 'SKIP_STACK_IN_SMALL is no longer needed as the backend can optimize it directly'],
   ['SAFE_STACK', [0], 'Replace SAFE_STACK=1 with STACK_OVERFLOW_CHECK=2'],
-  ['MEMORY_GROWTH_STEP', 'MEMORY_GROWTH_LINEAR_STEP']
+  ['MEMORY_GROWTH_STEP', 'MEMORY_GROWTH_LINEAR_STEP'],
+  // WASM_OBJECT_FILES is handled in emcc.py, supporting both 0 and 1 for now.
+  ['WASM_OBJECT_FILES', [0, 1], 'For LTO, use -flto or -fto=thin instead; to disable LTO, just do not pass WASM_OBJECT_FILES=1 as 1 is the default anyhow'],
+  ['TOTAL_MEMORY', 'INITIAL_MEMORY'],
+  ['WASM_MEM_MAX', 'MAXIMUM_MEMORY'],
+  ['BINARYEN_MEM_MAX', 'MAXIMUM_MEMORY'],
 ];
