@@ -10,6 +10,16 @@ var STACK_ALIGN = {{{ STACK_ALIGN }}};
 #if ASSERTIONS
 // stack management, and other functionality that is provided by the compiled code,
 // should not be used before it is ready
+
+#if DECLARE_ASM_MODULE_EXPORTS // These functions should not be defined with 'var' if DECLARE_ASM_MODULE_EXPORTS==0 (or the assignments won't be seen)
+/** @suppress{duplicate} */
+var stackSave;
+/** @suppress{duplicate} */
+var stackRestore;
+/** @suppress{duplicate} */
+var stackAlloc;
+#endif
+
 stackSave = stackRestore = stackAlloc = function() {
   abort('cannot use the stack before compiled code is ready to run, and has provided stack access');
 };
@@ -28,13 +38,9 @@ function dynamicAlloc(size) {
 #endif
   var ret = HEAP32[DYNAMICTOP_PTR>>2];
   var end = (ret + size + 15) & -16;
-  if (end > _emscripten_get_heap_size()) {
 #if ASSERTIONS
-    abort('failure to dynamicAlloc - memory growth etc. is not supported there, call malloc/sbrk directly');
-#else
-    abort();
+  assert(end <= HEAP8.length, 'failure to dynamicAlloc - memory growth etc. is not supported there, call malloc/sbrk directly');
 #endif
-  }
   HEAP32[DYNAMICTOP_PTR>>2] = end;
   return ret;
 }
@@ -176,12 +182,12 @@ function loadDynamicLibrary(lib, flags) {
   LDSO.loadedLibNames[lib] = handle;
   LDSO.loadedLibs[handle] = dso;
 
-  // libData <- lib
-  function loadLibData() {
+  // libData <- libFile
+  function loadLibData(libFile) {
 #if WASM
     // for wasm, we can use fetch for async, but for fs mode we can only imitate it
     if (flags.fs) {
-      var libData = flags.fs.readFile(lib, {encoding: 'binary'});
+      var libData = flags.fs.readFile(libFile, {encoding: 'binary'});
       if (!(libData instanceof Uint8Array)) {
         libData = new Uint8Array(lib_data);
       }
@@ -189,17 +195,17 @@ function loadDynamicLibrary(lib, flags) {
     }
 
     if (flags.loadAsync) {
-      return fetchBinary(lib);
+      return fetchBinary(libFile);
     }
     // load the binary synchronously
-    return readBinary(lib);
+    return readBinary(libFile);
 #else
     // for js we only imitate async for both native & fs modes.
     var libData;
     if (flags.fs) {
-      libData = flags.fs.readFile(lib, {encoding: 'utf8'});
+      libData = flags.fs.readFile(libFile, {encoding: 'utf8'});
     } else {
-      libData = read_(lib);
+      libData = read_(libFile);
     }
     return flags.loadAsync ? Promise.resolve(libData) : libData;
 #endif
@@ -210,7 +216,7 @@ function loadDynamicLibrary(lib, flags) {
 #if WASM
     return loadWebAssemblyModule(libData, flags)
 #else
-    var libModule = eval(libData)(
+    var libModule = /**@type{function(...)}*/(eval(libData))(
       alignFunctionTables(),
       Module
     );
@@ -607,6 +613,7 @@ Module['loadWebAssemblyModule'] = loadWebAssemblyModule;
 
 #if EMULATED_FUNCTION_POINTERS
 #if WASM == 0
+/** @param {Object=} module */
 function getFunctionTables(module) {
   if (!module) module = Module;
   var tables = {};
@@ -619,6 +626,7 @@ function getFunctionTables(module) {
   return tables;
 }
 
+/** @param {Object=} module */
 function alignFunctionTables(module) {
   var tables = getFunctionTables(module);
   var maxx = 0;
@@ -693,62 +701,13 @@ function getFuncWrapper(func, sig) {
   return sigCache[func];
 }
 
-#if RUNTIME_DEBUG
-var runtimeDebug = true; // Switch to false at runtime to disable logging at the right times
-
-var printObjectList = [];
-
-function prettyPrint(arg) {
-  if (typeof arg == 'undefined') return '!UNDEFINED!';
-  if (typeof arg == 'boolean') arg = arg + 0;
-  if (!arg) return arg;
-  var index = printObjectList.indexOf(arg);
-  if (index >= 0) return '<' + arg + '|' + index + '>';
-  if (arg.toString() == '[object HTMLImageElement]') {
-    return arg + '\n\n';
-  }
-  if (arg.byteLength) {
-    return '{' + Array.prototype.slice.call(arg, 0, Math.min(arg.length, 400)) + '}'; // Useful for correct arrays, less so for compiled arrays, see the code below for that
-    var buf = new ArrayBuffer(32);
-    var i8buf = new Int8Array(buf);
-    var i16buf = new Int16Array(buf);
-    var f32buf = new Float32Array(buf);
-    switch(arg.toString()) {
-      case '[object Uint8Array]':
-        i8buf.set(arg.subarray(0, 32));
-        break;
-      case '[object Float32Array]':
-        f32buf.set(arg.subarray(0, 5));
-        break;
-      case '[object Uint16Array]':
-        i16buf.set(arg.subarray(0, 16));
-        break;
-      default:
-        alert('unknown array for debugging: ' + arg);
-        throw 'see alert';
-    }
-    var ret = '{' + arg.byteLength + ':\n';
-    var arr = Array.prototype.slice.call(i8buf);
-    ret += 'i8:' + arr.toString().replace(/,/g, ',') + '\n';
-    arr = Array.prototype.slice.call(f32buf, 0, 8);
-    ret += 'f32:' + arr.toString().replace(/,/g, ',') + '}';
-    return ret;
-  }
-  if (typeof arg == 'object') {
-    printObjectList.push(arg);
-    return '<' + arg + '|' + (printObjectList.length-1) + '>';
-  }
-  if (typeof arg == 'number') {
-    if (arg > 0) return '0x' + arg.toString(16) + ' (' + arg + ')';
-  }
-  return arg;
-}
-#endif
+#include "runtime_debug.js"
 
 function makeBigInt(low, high, unsigned) {
   return unsigned ? ((+((low>>>0)))+((+((high>>>0)))*4294967296.0)) : ((+((low>>>0)))+((+((high|0)))*4294967296.0));
 }
 
+/** @param {Array=} args */
 function dynCall(sig, ptr, args) {
   if (args && args.length) {
 #if ASSERTIONS
@@ -815,12 +774,6 @@ GLOBAL_BASE = alignMemory(GLOBAL_BASE, {{{ MAX_GLOBAL_ALIGN || 1 }}});
 #endif
 
 #if WASM_BACKEND && USE_PTHREADS
-// The wasm backend path does not have a way to set the stack max, so ignore
-// the stack max parameter, this function only resets the stack base.
-function establishStackSpace(base/*, max*/) {
-  stackRestore(base);
-}
-
 // JS library code refers to Atomics in the manner used from asm.js, provide
 // the same API here.
 var Atomics_load = Atomics.load;
