@@ -102,16 +102,17 @@ typedef union em_variant_val
   char *cp;
 } em_variant_val;
 
+// Proxied C/C++ functions support at most this many arguments. Dispatch is static/strongly typed by signature.
 #define EM_QUEUED_CALL_MAX_ARGS 11
+// Proxied JS function can support a few more arguments than proxied C/C++ functions, because the dispatch
+// is variadic and signature independent.
+#define EM_QUEUED_JS_CALL_MAX_ARGS 20
 typedef struct em_queued_call
 {
   int functionEnum;
   void *functionPtr;
   int operationDone;
-  // If set, this is a JS call: number of arguments are in functionEnum, the index in functionPtr,
-  // and args in args.
-  int js;
-  em_variant_val args[EM_QUEUED_CALL_MAX_ARGS];
+  em_variant_val args[EM_QUEUED_JS_CALL_MAX_ARGS];
   em_variant_val returnValue;
 
   // An optional pointer to a secondary data block that should be free()d when this queued call is freed.
@@ -196,6 +197,10 @@ typedef int (*em_func_iiiiiiiiii)(int, int, int, int, int, int, int, int, int);
 #define EM_FUNC_SIG_PARAM_D   0x3U
 #define EM_FUNC_SIG_SET_PARAM(i, type) ((EM_FUNC_SIGNATURE)(type) << (2*i))
 
+// Extra types used in WebGL glGet*() calls (not used in proxying)
+#define EM_FUNC_SIG_PARAM_B   0x4U
+#define EM_FUNC_SIG_PARAM_F2I 0x5U
+
 // In total, the above encoding scheme gives the following 32-bit structure for the proxied function signatures (highest -> lowest bit order):
 // RRRiiiiSbbaa99887766554433221100
 // where RRR is return type
@@ -247,9 +252,9 @@ typedef int (*em_func_iiiiiiiiii)(int, int, int, int, int, int, int, int, int);
 #define EM_PROXIED_FUNC_SPECIAL(x) (EM_FUNC_SIG_SPECIAL_INTERNAL | ((x) << 20))
 
 #define EM_PROXIED_PTHREAD_CREATE (EM_PROXIED_FUNC_SPECIAL(0) | EM_FUNC_SIG_IIIII)
-#define EM_PROXIED_SYSCALL (EM_PROXIED_FUNC_SPECIAL(1) | EM_FUNC_SIG_III)
 #define EM_PROXIED_CREATE_CONTEXT (EM_PROXIED_FUNC_SPECIAL(2) | EM_FUNC_SIG_III)
 #define EM_PROXIED_RESIZE_OFFSCREENCANVAS (EM_PROXIED_FUNC_SPECIAL(3) | EM_FUNC_SIG_IIII)
+#define EM_PROXIED_JS_FUNCTION (EM_PROXIED_FUNC_SPECIAL(4) | EM_FUNC_SIG_D)
 
 // Runs the given function synchronously on the main Emscripten runtime thread.
 // If this thread is the main thread, the operation is immediately performed, and the result is returned.
@@ -305,9 +310,6 @@ void emscripten_current_thread_process_queued_calls(void);
 
 pthread_t emscripten_main_browser_thread_id(void);
 
-// Direct syscall access, second argument is a varargs pointer. used in proxying
-int emscripten_syscall(int, void*);
-
 // Synchronously sleeps the calling thread for the given number of milliseconds.
 // Note: Calling this on the main browser thread is _very_ _very_ bad for application logic throttling,
 // because it does not save any battery, it will spin up the CPU at 100%, lock up the UI, printfs will not come
@@ -366,6 +368,10 @@ struct thread_profiler_block
 	char name[32];
 };
 
+// Called when blocking on the main thread. This will error if main thread
+// blocking is not enabled, see ALLOW_BLOCKING_ON_MAIN_THREAD.
+void emscripten_check_blocking_allowed(void);
+
 #ifdef __cplusplus
 }
 #endif
