@@ -112,12 +112,19 @@ function nullify(node) {
   node.raw = 'null';
 }
 
-function undefinedify(node) {
+// This converts the node into something that terser will ignore in a var
+// declaration, that is, it is a way to get rid of initial values.
+function convertToNothingInVarInit(node) {
   node.type = 'Literal';
   node.value = undefined;
   node.raw = 'undefined';
 }
 
+function convertToNull(node) {
+  node.type = 'Identifier';
+  node.name = 'null';
+}
+
 function setLiteralValue(item, value) {
   item.value = value;
   item.raw = "'" + value + "'";
@@ -550,7 +557,7 @@ function emitDCEGraph(ast) {
   //
   // The imports that wasm receives look like this:
   //
-  //  Module.asmLibraryArg = { "abort": abort, "assert": assert, [..] };
+  //  var asmLibraryArg = { "abort": abort, "assert": assert, [..] };
   //
   // The exports are trickier, as they have a different form whether or not
   // async compilation is enabled. It can be either:
@@ -563,6 +570,15 @@ function emitDCEGraph(ast) {
   //   return Module["asm"]["_malloc"].apply(null, arguments);
   //  });
   //
+  // or, in the minimal runtime, it looks like
+  //
+  //  WebAssembly.instantiate(Module["wasm"], imports).then(function(output) {
+  //   var asm = output.instance.exports; // may also not have "var", if
+  //                                      // declared outside and used elsewhere
+  //   ..
+  //   _malloc = asm["malloc"];
+  //   ..
+  //  });
   var imports = [];
   var defuns = [];
   var dynCallNames = [];
@@ -570,6 +586,7 @@ function emitDCEGraph(ast) {
   var modulePropertyToGraphName = {};
   var exportNameToGraphName = {}; // identical to asm['..'] nameToGraphName
   var foundAsmLibraryArgAssign = false;
+  var foundMinimalRuntimeExports = false;
   var graph = [];
 
   function saveAsmExport(name, asmName) {
@@ -628,15 +645,78 @@ function emitDCEGraph(ast) {
               // name may be slightly different (extra "_" in wasm backend)
               saveAsmExport(name, asmName);
               emptyOut(node); // ignore this in the second pass; this does not root
+              return;
             }
           }
         }
       }
+      // A variable declaration that has no initial values can be ignored in
+      // the second pass, these are just declarations, not roots - an actual
+      // use must be found in order to root.
+      if (!node.declarations.reduce(function(hasInit, decl) {
+            return hasInit || !!decl.init
+          }, false)) {
+        emptyOut(node);
+      }
     } else if (node.type === 'FunctionDeclaration') {
       defuns.push(node);
       var name = node.id.name;
       nameToGraphName[name] = getGraphName(name, 'defun');
       emptyOut(node); // ignore this in the second pass; we scan defuns separately
+    } else if (node.type === 'FunctionExpression') {
+      // Check if this is the minimal runtime exports function, which looks like
+      //   function(output) { var asm = output.instance.exports;
+      if (node.params.length === 1 && node.params[0].type === 'Identifier' &&
+          node.params[0].name === 'output' && node.body.type === 'BlockStatement') {
+        var body = node.body.body;
+        if (body.length >= 1) {
+          var first = body[0];
+          var target, value; // "(var?) target = value"
+          // Look either for  var asm =  or just   asm =
+          if (first.type === 'VariableDeclaration' && first.declarations.length === 1) {
+            var decl = first.declarations[0];
+            target = decl.id;
+            value = decl.init;
+          } else if (first.type === 'ExpressionStatement' &&
+                     first.expression.type === 'AssignmentExpression') {
+            var assign = first.expression;
+            if (assign.operator === '=') {
+              target = assign.left;
+              value = assign.right;
+            }
+          }
+          if (target.type === 'Identifier' && target.name === 'asm' && value) {
+            if (value.type === 'MemberExpression' &&
+                value.object.type === 'MemberExpression' &&
+                value.object.object.type === 'Identifier' &&
+                value.object.object.name === 'output' &&
+                value.object.property.type === 'Identifier' &&
+                value.object.property.name === 'instance' &&
+                value.property.type === 'Identifier' &&
+                value.property.name === 'exports') {
+              // This looks very much like what we are looking for.
+              assert(!foundMinimalRuntimeExports);
+              for (var i = 1; i < body.length; i++) {
+                var item = body[i];
+                if (item.type === 'ExpressionStatement' &&
+                    item.expression.type === 'AssignmentExpression' &&
+                    item.expression.operator === '=' &&
+                    item.expression.left.type === 'Identifier' &&
+                    item.expression.right.type === 'MemberExpression' &&
+                    item.expression.right.object.type === 'Identifier' &&
+                    item.expression.right.object.name === 'asm' &&
+                    item.expression.right.property.type === 'Literal') {
+                  var name = item.expression.left.name;
+                  var asmName = item.expression.right.property.value;
+                  saveAsmExport(name, asmName);
+                  emptyOut(item); // ignore all this in the second pass; this does not root
+                }
+              }
+              foundMinimalRuntimeExports = true;
+            }
+          }
+        }
+      }
     }
   });
   // must find the info we need
@@ -661,7 +741,7 @@ function emitDCEGraph(ast) {
       import: ['env', import_],
       reaches: {}
     };
-    if (nameToGraphName.hasOwnProperty(import_)) {
+    if (Object.prototype.hasOwnProperty.call(nameToGraphName, import_)) {
       info.reaches[nameToGraphName[import_]] = 1;
     } // otherwise, it's a number, ignore
   });
@@ -681,12 +761,12 @@ function emitDCEGraph(ast) {
     var reached;
     if (node.type === 'Identifier') {
       var name = node.name;
-      if (nameToGraphName.hasOwnProperty(name)) {
+      if (Object.prototype.hasOwnProperty.call(nameToGraphName, name)) {
         reached = nameToGraphName[name];
       }
     } else if (isModuleUse(node)) {
       var name = getAsmOrModuleUseName(node);
-      if (modulePropertyToGraphName.hasOwnProperty(name)) {
+      if (Object.prototype.hasOwnProperty.call(modulePropertyToGraphName, name)) {
         reached = modulePropertyToGraphName[name];
       }
     } else if (isStaticDynCall(node)) {
@@ -697,7 +777,7 @@ function emitDCEGraph(ast) {
     } else if (isAsmUse(node)) {
       // any remaining asm uses are always rooted in any case
       var name = getAsmOrModuleUseName(node);
-      if (exportNameToGraphName.hasOwnProperty(name)) {
+      if (Object.prototype.hasOwnProperty.call(exportNameToGraphName, name)) {
         infos[exportNameToGraphName[name]].root = true;
       }
       return;
@@ -778,7 +858,24 @@ function applyDCEGraphRemovals(ast) {
         var value = node.right;
         if ((full in unused) &&
             (isAsmUse(value) || !hasSideEffects(value))) {
-          undefinedify(node);
+          // This will be in a var init, and we just remove that value.
+          convertToNothingInVarInit(node);
+        }
+      }
+    } else if (node.type === 'ExpressionStatement') {
+      var expr = node.expression;
+      // In the minimal runtime code pattern we have just
+      //   x = asm['x']
+      // and never in a var.
+      if (expr.operator === '=' &&
+          expr.left.type === 'Identifier' &&
+          isAsmUse(expr.right)) {
+        var name = expr.left.name;
+        if (name === getAsmOrModuleUseName(expr.right)) {
+          var full = 'emcc$export$' + name;
+          if (full in unused) {
+            emptyOut(node);
+          }
         }
       }
     }
@@ -937,12 +1034,14 @@ function reattachComments(ast, comments) {
 // Main
 
 var arguments = process['argv'].slice(2);;
-var preserveComments = arguments.indexOf('--preserveComments');
-if (preserveComments > -1) {
-  arguments.splice(preserveComments, 1);
-  preserveComments = true;
+// If enabled, output retains parentheses and comments so that the
+// output can further be passed out to Closure.
+var closureFriendly = arguments.indexOf('--closureFriendly');
+if (closureFriendly > -1) {
+  arguments.splice(closureFriendly, 1);
+  closureFriendly = true;
 } else {
-  preserveComments = false;  
+  closureFriendly = false;
 }
 
 var infile = arguments[0];
@@ -955,9 +1054,9 @@ if (extraInfoStart > 0) {
   extraInfo = JSON.parse(input.substr(extraInfoStart + 14));
 }
 // Collect all JS code comments to this array so that we can retain them in the outputted code
-// if --preserveComments was requested.
+// if --closureFriendly was requested.
 var sourceComments = [];
-var ast = acorn.parse(input, { ecmaVersion: 6, onComment: preserveComments ? sourceComments : undefined });
+var ast = acorn.parse(input, { ecmaVersion: 6, preserveParens: closureFriendly, onComment: closureFriendly ? sourceComments : undefined });
 
 var minifyWhitespace = false;
 var noPrint = false;
@@ -981,7 +1080,7 @@ passes.forEach(function(pass) {
 if (!noPrint) {
   var terserAst = terser.AST_Node.from_mozilla_ast(ast);
 
-  if (preserveComments) {
+  if (closureFriendly) {
     reattachComments(terserAst, sourceComments);
   }
 
