@@ -1,29 +1,36 @@
+# Copyright 2018 The Emscripten Authors.  All rights reserved.
+# Emscripten is available under two separate licenses, the MIT license and the
+# University of Illinois/NCSA Open Source License.  Both these licenses can be
+# found in the LICENSE file.
+
 """Enables colored logger just by importing this module
 """
 
+import ctypes
 import sys
 import logging
 
-def add_coloring_to_emit_windows(fn):
-  def _out_handle(self):
-    import ctypes
-    return ctypes.windll.kernel32.GetStdHandle(self.STD_OUTPUT_HANDLE)
-  out_handle = property(_out_handle)
 
-  def _get_color(self):
-    import ctypes
+def add_coloring_to_emit_windows(fn):
+  # Constants from the Windows API
+  STD_OUTPUT_HANDLE = -11
+
+  def _get_color():
     SHORT = ctypes.c_short
     WORD = ctypes.c_ushort
+
     class COORD(ctypes.Structure):
       _fields_ = [
         ("X", SHORT),
         ("Y", SHORT)]
+
     class SMALL_RECT(ctypes.Structure):
       _fields_ = [
         ("Left", SHORT),
         ("Top", SHORT),
         ("Right", SHORT),
         ("Bottom", SHORT)]
+
     class CONSOLE_SCREEN_BUFFER_INFO(ctypes.Structure):
       _fields_ = [
         ("dwSize", COORD),
@@ -31,53 +38,42 @@ def add_coloring_to_emit_windows(fn):
         ("wAttributes", WORD),
         ("srWindow", SMALL_RECT),
         ("dwMaximumWindowSize", COORD)]
-    # Constants from the Windows API
-    self.STD_OUTPUT_HANDLE = -11
-    hdl = ctypes.windll.kernel32.GetStdHandle(self.STD_OUTPUT_HANDLE)
+
+    hdl = ctypes.windll.kernel32.GetStdHandle(STD_OUTPUT_HANDLE)
     csbi = CONSOLE_SCREEN_BUFFER_INFO()
     ctypes.windll.kernel32.GetConsoleScreenBufferInfo(hdl, ctypes.byref(csbi))
     return csbi.wAttributes
 
-  def _set_color(self, code):
-    import ctypes
-    # Constants from the Windows API
-    self.STD_OUTPUT_HANDLE = -11
-    hdl = ctypes.windll.kernel32.GetStdHandle(self.STD_OUTPUT_HANDLE)
+  def _set_color(code):
+    hdl = ctypes.windll.kernel32.GetStdHandle(STD_OUTPUT_HANDLE)
     ctypes.windll.kernel32.SetConsoleTextAttribute(hdl, code)
 
-  setattr(logging.StreamHandler, '_get_color', _get_color)
-  setattr(logging.StreamHandler, '_set_color', _set_color)
-
   def new(*args):
-    FOREGROUND_BLUE      = 0x0001 # text color contains blue.
-    FOREGROUND_GREEN     = 0x0002 # text color contains green.
-    FOREGROUND_RED       = 0x0004 # text color contains red.
-    FOREGROUND_INTENSITY = 0x0008 # text color is intensified.
-    FOREGROUND_WHITE     = FOREGROUND_BLUE|FOREGROUND_GREEN |FOREGROUND_RED
-    # winbase.h
-    STD_INPUT_HANDLE = -10
-    STD_OUTPUT_HANDLE = -11
-    STD_ERROR_HANDLE = -12
+    FOREGROUND_BLUE      = 0x0001 # noqa; text color contains blue.
+    FOREGROUND_GREEN     = 0x0002 # noqa; text color contains green.
+    FOREGROUND_RED       = 0x0004 # noqa; text color contains red.
+    FOREGROUND_INTENSITY = 0x0008 # noqa; text color is intensified.
+    FOREGROUND_WHITE     = FOREGROUND_BLUE|FOREGROUND_GREEN |FOREGROUND_RED # noqa
 
     # wincon.h
-    FOREGROUND_BLACK     = 0x0000
-    FOREGROUND_BLUE      = 0x0001
-    FOREGROUND_GREEN     = 0x0002
-    FOREGROUND_CYAN      = 0x0003
-    FOREGROUND_RED       = 0x0004
-    FOREGROUND_MAGENTA   = 0x0005
-    FOREGROUND_YELLOW    = 0x0006
-    FOREGROUND_GREY      = 0x0007
+    FOREGROUND_BLACK     = 0x0000 # noqa
+    FOREGROUND_BLUE      = 0x0001 # noqa
+    FOREGROUND_GREEN     = 0x0002 # noqa
+    FOREGROUND_CYAN      = 0x0003 # noqa
+    FOREGROUND_RED       = 0x0004 # noqa
+    FOREGROUND_MAGENTA   = 0x0005 # noqa
+    FOREGROUND_YELLOW    = 0x0006 # noqa
+    FOREGROUND_GREY      = 0x0007 # noqa
     FOREGROUND_INTENSITY = 0x0008 # foreground color is intensified.
 
-    BACKGROUND_BLACK     = 0x0000
-    BACKGROUND_BLUE      = 0x0010
-    BACKGROUND_GREEN     = 0x0020
-    BACKGROUND_CYAN      = 0x0030
-    BACKGROUND_RED       = 0x0040
-    BACKGROUND_MAGENTA   = 0x0050
-    BACKGROUND_YELLOW    = 0x0060
-    BACKGROUND_GREY      = 0x0070
+    BACKGROUND_BLACK     = 0x0000 # noqa
+    BACKGROUND_BLUE      = 0x0010 # noqa
+    BACKGROUND_GREEN     = 0x0020 # noqa
+    BACKGROUND_CYAN      = 0x0030 # noqa
+    BACKGROUND_RED       = 0x0040 # noqa
+    BACKGROUND_MAGENTA   = 0x0050 # noqa
+    BACKGROUND_YELLOW    = 0x0060 # noqa
+    BACKGROUND_GREY      = 0x0070 # noqa
     BACKGROUND_INTENSITY = 0x0080 # background color is intensified.
     levelno = args[1].levelno
     if(levelno >= 50):
@@ -91,39 +87,48 @@ def add_coloring_to_emit_windows(fn):
     elif(levelno >= 10):
         color = FOREGROUND_MAGENTA
     else:
-        color =  FOREGROUND_WHITE
-    old_color = args[0]._get_color()
-    args[0]._set_color(color)
+        color = FOREGROUND_WHITE
+    old_color = _get_color()
+    _set_color(color)
     ret = fn(*args)
-    args[0]._set_color(old_color)
-    #print "after"
+    _set_color(old_color)
     return ret
+
+  new.orig_func = fn
   return new
 
+
 def add_coloring_to_emit_ansi(fn):
   # add methods we need to the class
   def new(*args):
     levelno = args[1].levelno
-    if(levelno >= 50):
+    if levelno >= 50:
       color = '\x1b[31m' # red
-    elif(levelno >= 40):
+    elif levelno >= 40:
       color = '\x1b[31m' # red
-    elif(levelno >= 30):
+    elif levelno >= 30:
       color = '\x1b[33m' # yellow
-    elif(levelno >= 20):
+    elif levelno >= 20:
       color = '\x1b[32m' # green
-    elif(levelno >= 10):
+    elif levelno >= 10:
       color = '\x1b[35m' # pink
     else:
       color = '\x1b[0m' # normal
-    args[1].msg = color + args[1].msg +  '\x1b[0m'  # normal
-    #print "after"
+    args[1].msg = color + args[1].msg + '\x1b[0m'  # normal
     return fn(*args)
+
+  new.orig_func = fn
   return new
 
+
 def enable():
   if sys.stderr.isatty():
     if sys.platform.startswith('win'):
       logging.StreamHandler.emit = add_coloring_to_emit_windows(logging.StreamHandler.emit)
     else:
       logging.StreamHandler.emit = add_coloring_to_emit_ansi(logging.StreamHandler.emit)
+
+
+def disable():
+  if hasattr(logging.StreamHandler.emit, 'orig_func'):
+    logging.StreamHandler.emit = logging.StreamHandler.emit.orig_func
