@@ -97,7 +97,7 @@ QUIET = (__name__ != '__main__')
 
 def show(msg):
   if shared.DEBUG or not QUIET:
-    sys.stderr.write('gen_struct_info: ' + msg + '\n')
+    sys.stderr.write('gen_struct_info: %s\n' % msg)
 
 
 # Try to load pycparser.
@@ -344,8 +344,6 @@ def gen_inspect_code(path, struct, code):
 
 
 def inspect_code(headers, cpp_opts, structs, defines):
-  show('Generating C code...')
-
   code = ['#include <stdio.h>', '#include <stddef.h>']
   # Include all the needed headers.
   for path in headers:
@@ -378,10 +376,11 @@ def inspect_code(headers, cpp_opts, structs, defines):
 
   # Write the source code to a temporary file.
   src_file = tempfile.mkstemp('.c')
-  js_file = tempfile.mkstemp('.js')
-
+  show('Generating C code... ' + src_file[1])
   os.write(src_file[0], shared.asbytes('\n'.join(code)))
 
+  js_file = tempfile.mkstemp('.js')
+
   # Close all unneeded FDs.
   os.close(src_file[0])
   os.close(js_file[0])
@@ -404,30 +403,29 @@ def inspect_code(headers, cpp_opts, structs, defines):
   cmd = [shared.PYTHON, shared.EMCC] + cpp_opts + ['-o', js_file[1], src_file[1], '-s', 'BOOTSTRAPPING_STRUCT_INFO=1', '-s', 'WARN_ON_UNDEFINED_SYMBOLS=0', '-O0', '--js-opts', '0', '--memory-init-file', '0', '-s', 'SINGLE_FILE=1', '-Wno-format']
   if not shared.Settings.WASM_BACKEND:
     cmd += ['-s', 'WASM=0']
-  if not shared.Settings.WASM_OBJECT_FILES:
-    cmd += ['-s', 'WASM_OBJECT_FILES=0']
+  if shared.Settings.LTO:
+    cmd += ['-flto=' + shared.Settings.LTO]
 
+  show(cmd)
   try:
-    try:
-      subprocess.check_call(cmd, env=safe_env)
-    except subprocess.CalledProcessError:
-      sys.stderr.write('FAIL: Compilation failed!\n')
-      sys.exit(1)
+    subprocess.check_call(cmd, env=safe_env)
+  except subprocess.CalledProcessError:
+    sys.stderr.write('FAIL: Compilation failed!\n')
+    sys.exit(1)
 
-    # Run the compiled program.
-    show('Calling generated program...')
-    try:
-      info = shared.run_js(js_file[1]).splitlines()
-    except subprocess.CalledProcessError:
-      sys.stderr.write('FAIL: Running the generated program failed!\n')
-      sys.exit(1)
+  # Run the compiled program.
+  show('Calling generated program... ' + js_file[1])
+  try:
+    info = shared.run_js(js_file[1]).splitlines()
+  except subprocess.CalledProcessError:
+    sys.stderr.write('FAIL: Running the generated program failed!\n')
+    sys.exit(1)
 
-  finally:
-    # Remove all temporary files.
-    os.unlink(src_file[1])
+  # Remove all temporary files.
+  os.unlink(src_file[1])
 
-    if os.path.exists(js_file[1]):
-      os.unlink(js_file[1])
+  if os.path.exists(js_file[1]):
+    os.unlink(js_file[1])
 
   # Parse the output of the program into a dict.
   return parse_c_output(info)
@@ -471,6 +469,7 @@ def output_json(obj, compressed=True, stream=None):
   else:
     json.dump(obj, stream, indent=4, sort_keys=True)
 
+  stream.write('\n')
   stream.close()
 
 
