@@ -19,7 +19,7 @@ from glob import iglob
 
 from . import ports
 from . import shared
-from tools.shared import check_call, mangle_c_symbol_name, demangle_c_symbol_name
+from tools.shared import mangle_c_symbol_name, demangle_c_symbol_name
 
 stdout = None
 stderr = None
@@ -59,35 +59,36 @@ def dir_is_newer(dir_a, dir_b):
 def get_cflags(force_object_files=False):
   flags = []
   if shared.Settings.WASM_BACKEND:
-    if force_object_files:
-      flags += ['-s', 'WASM_OBJECT_FILES=1']
-    elif not shared.Settings.WASM_OBJECT_FILES:
-      flags += ['-s', 'WASM_OBJECT_FILES=0']
+    if shared.Settings.LTO and not force_object_files:
+      flags += ['-flto=' + shared.Settings.LTO]
   if shared.Settings.RELOCATABLE:
     flags += ['-s', 'RELOCATABLE']
   return flags
 
 
-def run_build_command(cmd):
-  # this must only be called on a standard build command
-  assert cmd[0] == shared.PYTHON and cmd[1] in (shared.EMCC, shared.EMXX)
-  # add standard cflags, but also allow the cmd to override them
-  cmd = cmd[:2] + get_cflags() + cmd[2:]
+def run_one_command(cmd):
+  # Helper function used by run_build_commands.
+  if shared.EM_BUILD_VERBOSE:
+    print(' '.join(cmd))
   shared.run_process(cmd, stdout=stdout, stderr=stderr)
 
 
-def run_commands(commands):
+def run_build_commands(commands):
   cores = min(len(commands), shared.Building.get_num_cores())
   if cores <= 1:
     for command in commands:
-      run_build_command(command)
+      run_one_command(command)
   else:
     pool = shared.Building.get_multiprocessing_pool()
     # https://stackoverflow.com/questions/1408356/keyboard-interrupts-with-pythons-multiprocessing-pool
     # https://bugs.python.org/issue8296
     # 999999 seconds (about 11 days) is reasonably huge to not trigger actual timeout
     # and is smaller than the maximum timeout value 4294967.0 for Python 3 on Windows (threading.TIMEOUT_MAX)
-    pool.map_async(run_build_command, commands, chunksize=1).get(999999)
+    pool.map_async(run_one_command, commands, chunksize=1).get(999999)
+
+
+def static_library_ext():
+  return '.a' if shared.Settings.WASM_BACKEND else '.bc'
 
 
 def create_lib(libname, inputs):
@@ -146,7 +147,19 @@ def get_wasm_libc_rt_files():
     path_components=['system', 'lib', 'libc'],
     filenames=['emscripten_memcpy.c', 'emscripten_memset.c',
                'emscripten_memmove.c'])
-  return math_files + other_files
+  # Calls to iprintf can be generated during codegen. Ideally we wouldn't
+  # compile these with -O2 like we do the rest of compiler-rt since its
+  # probably not performance sensitive.  However we don't currently have
+  # a way to set per-file compiler flags.  And hopefully we should be able
+  # move all this stuff back into libc once we it LTO compatible.
+  iprintf_files = files_in_path(
+    path_components=['system', 'lib', 'libc', 'musl', 'src', 'stdio'],
+    filenames=['__towrite.c', '__overflow.c', 'fwrite.c', 'fputs.c',
+               'printf.c', 'puts.c', '__lockfile.c'])
+  iprintf_files += files_in_path(
+    path_components=['system', 'lib', 'libc', 'musl', 'src', 'string'],
+    filenames=['strlen.c'])
+  return math_files + other_files + iprintf_files
 
 
 class Library(object):
@@ -259,7 +272,7 @@ class Library(object):
   src_glob = None
   src_glob_exclude = None
 
-  # Whether to always generate WASM object files, even though WASM_OBJECT_FILES=0.
+  # Whether to always generate WASM object files, even when LTO is set
   force_object_files = False
 
   def __init__(self):
@@ -362,7 +375,7 @@ class Library(object):
       o = self.in_temp(os.path.basename(src) + '.o')
       commands.append([shared.PYTHON, self.emcc, '-c', src, '-o', o] + cflags)
       objects.append(o)
-    run_commands(commands)
+    run_build_commands(commands)
     return objects
 
   def build(self):
@@ -414,7 +427,7 @@ class Library(object):
     """
     Return the appropriate file extension for this library.
     """
-    return '.a' if shared.Settings.WASM_BACKEND else '.bc'
+    return static_library_ext()
 
   def get_filename(self):
     """
@@ -559,13 +572,12 @@ class NoExceptLibrary(Library):
 
   @classmethod
   def get_default_variation(cls, **kwargs):
-    return super(NoExceptLibrary, cls).get_default_variation(is_noexcept=shared.Settings.DISABLE_EXCEPTION_CATCHING, **kwargs)
+    return super(NoExceptLibrary, cls).get_default_variation(is_noexcept=shared.Settings.DISABLE_EXCEPTION_CATCHING == 1, **kwargs)
 
 
 class MuslInternalLibrary(Library):
   includes = [
     ['system', 'lib', 'libc', 'musl', 'src', 'internal'],
-    ['system', 'lib', 'libc', 'musl', 'arch', 'js'],
   ]
 
   cflags = [
@@ -626,20 +638,12 @@ class libcompiler_rt(Library):
 
   cflags = ['-O2', '-fno-builtin']
   src_dir = ['system', 'lib', 'compiler-rt', 'lib', 'builtins']
-  src_files = ['divdc3.c', 'divsc3.c', 'muldc3.c', 'mulsc3.c']
   if shared.Settings.WASM_BACKEND:
-    src_files += ['addtf3.c', 'ashlti3.c', 'ashrti3.c', 'atomic.c', 'comparetf2.c', 'clzti2.c',
-                  'divtf3.c', 'divti3.c', 'udivmodti4.c',
-                  'extenddftf2.c', 'extendsftf2.c',
-                  'fixdfti.c', 'fixsfti.c', 'fixtfdi.c', 'fixtfsi.c', 'fixtfti.c',
-                  'fixunsdfti.c', 'fixunssfti.c', 'fixunstfdi.c', 'fixunstfsi.c', 'fixunstfti.c',
-                  'floatditf.c', 'floatsitf.c', 'floattidf.c', 'floattisf.c', 'floattitf.c',
-                  'floatunditf.c', 'floatunsitf.c', 'floatuntidf.c', 'floatuntisf.c', 'lshrti3.c',
-                  'modti3.c', 'multc3.c', 'multf3.c', 'multi3.c', 'subtf3.c', 'udivti3.c', 'umodti3.c', 'ashrdi3.c',
-                  'ashldi3.c', 'fixdfdi.c', 'floatdidf.c', 'lshrdi3.c', 'moddi3.c',
-                  'trunctfdf2.c', 'trunctfsf2.c', 'umoddi3.c', 'fixunsdfdi.c', 'muldi3.c',
-                  'divdi3.c', 'divmoddi4.c', 'udivdi3.c', 'udivmoddi4.c']
+    filelist = shared.path_from_root('system', 'lib', 'compiler-rt', 'filelist.txt')
+    src_files = open(filelist).read().splitlines()
     src_files.append(shared.path_from_root('system', 'lib', 'compiler-rt', 'extras.c'))
+  else:
+    src_files = ['divdc3.c', 'divsc3.c', 'muldc3.c', 'mulsc3.c']
 
 
 class libc(AsanInstrumentedLibrary, MuslInternalLibrary, MTLibrary):
@@ -682,7 +686,7 @@ class libc(AsanInstrumentedLibrary, MuslInternalLibrary, MTLibrary):
         'proto.c', 'gethostbyaddr.c', 'gethostbyaddr_r.c', 'gethostbyname.c',
         'gethostbyname2_r.c', 'gethostbyname_r.c', 'gethostbyname2.c',
         'usleep.c', 'alarm.c', 'syscall.c', '_exit.c', 'popen.c',
-        'getgrouplist.c', 'initgroups.c', 'wordexp.c', 'timer_create.c',
+        'getgrouplist.c', 'initgroups.c', 'timer_create.c',
         'faccessat.c',
     ]
 
@@ -736,9 +740,16 @@ class libc(AsanInstrumentedLibrary, MuslInternalLibrary, MTLibrary):
           if not cancel:
             libc_files.append(os.path.join(musl_srcdir, dirpath, f))
 
+    # Allowed files from blacklisted modules
+    libc_files += files_in_path(
+        path_components=['system', 'lib', 'libc', 'musl', 'src', 'time'],
+        filenames=['clock_settime.c'])
+    libc_files += files_in_path(
+        path_components=['system', 'lib', 'libc', 'musl', 'src', 'legacy'],
+        filenames=['getpagesize.c'])
+
     if shared.Settings.WASM_BACKEND:
       # See libc_extras below
-      libc_files.append(shared.path_from_root('system', 'lib', 'libc', 'extras.c'))
       # Include all the getenv stuff with the wasm backend. With fastcomp we
       # still use JS because libc is a .bc file and we don't want to have a
       # global constructor there for __environ, which would mean it is always
@@ -747,7 +758,13 @@ class libc(AsanInstrumentedLibrary, MuslInternalLibrary, MTLibrary):
           path_components=['system', 'lib', 'libc', 'musl', 'src', 'env'],
           filenames=['__environ.c', 'getenv.c', 'putenv.c', 'setenv.c', 'unsetenv.c'])
 
-    libc_files.append(shared.path_from_root('system', 'lib', 'libc', 'wasi-helpers.c'))
+    libc_files += files_in_path(
+        path_components=['system', 'lib', 'libc', 'musl', 'src', 'sched'],
+        filenames=['sched_yield.c'])
+
+    libc_files += files_in_path(
+        path_components=['system', 'lib', 'libc'],
+        filenames=['extras.c', 'wasi-helpers.c'])
 
     return libc_files
 
@@ -788,11 +805,12 @@ class libc_wasm(MuslInternalLibrary):
                'tan.c', 'tanf.c', 'tanl.c', 'acos.c', 'acosf.c', 'acosl.c',
                'asin.c', 'asinf.c', 'asinl.c', 'atan.c', 'atanf.c', 'atanl.c',
                'atan2.c', 'atan2f.c', 'atan2l.c', 'exp.c', 'expf.c', 'expl.c',
-               'log.c', 'logf.c', 'logl.c', 'pow.c', 'powf.c', 'powl.c']
+               'log.c', 'logf.c', 'logl.c', 'pow.c', 'powf.c', 'powl.c',
+               'sqrtl.c', 'ceill.c', 'floorl.c', 'fabsl.c']
 
   def can_use(self):
     # if building to wasm, we need more math code, since we have fewer builtins
-    return shared.Settings.WASM
+    return super(libc_wasm, self).can_use() and shared.Settings.WASM
 
 
 class crt1(MuslInternalLibrary):
@@ -807,30 +825,36 @@ class crt1(MuslInternalLibrary):
     return '.o'
 
   def can_use(self):
-    return shared.Settings.STANDALONE_WASM
+    return super(crt1, self).can_use() and shared.Settings.STANDALONE_WASM
 
   def can_build(self):
-    return shared.Settings.WASM_BACKEND
+    return super(crt1, self).can_build() and shared.Settings.WASM_BACKEND
 
 
 class libc_extras(MuslInternalLibrary):
   """This library is separate from libc itself for fastcomp only so that the
-  constructor it contains can be DCE'd.  With the wasm backend libc it is a .a
-  file so object file granularity applies.
+  constructor it contains can be DCE'd.  Such tricks are not needed wih the
+  the wasm backend because it uses .o file linking granularity.
   """
 
   name = 'libc-extras'
   src_dir = ['system', 'lib', 'libc']
-  src_files = ['extras.c']
+  src_files = ['extras_fastcomp.c']
 
   def can_build(self):
-    return not shared.Settings.WASM_BACKEND
+    return super(libc_extras, self).can_build() and not shared.Settings.WASM_BACKEND
 
 
 class libcxxabi(CXXLibrary, NoExceptLibrary, MTLibrary):
   name = 'libc++abi'
   depends = ['libc']
-  cflags = ['-std=c++11', '-Oz', '-D_LIBCPP_DISABLE_VISIBILITY_ANNOTATIONS']
+  cflags = [
+      '-Oz',
+      '-D_LIBCPP_DISABLE_VISIBILITY_ANNOTATIONS',
+      # Remove this once we update to include this llvm
+      # revision: https://reviews.llvm.org/D64961
+      '-D_LIBCXXABI_GUARD_ABI_ARM',
+    ]
   includes = [
     ['system', 'lib', 'libunwind', 'include'],
   ]
@@ -872,7 +896,7 @@ class libcxx(NoBCLibrary, CXXLibrary, NoExceptLibrary, MTLibrary):
   name = 'libc++'
   depends = ['libc++abi']
 
-  cflags = ['-std=c++11', '-DLIBCXX_BUILDING_LIBCXXABI=1', '-D_LIBCPP_BUILDING_LIBRARY', '-Oz',
+  cflags = ['-DLIBCXX_BUILDING_LIBCXXABI=1', '-D_LIBCPP_BUILDING_LIBRARY', '-Oz',
             '-D_LIBCPP_DISABLE_VISIBILITY_ANNOTATIONS']
 
   src_dir = ['system', 'lib', 'libcxx']
@@ -880,18 +904,21 @@ class libcxx(NoBCLibrary, CXXLibrary, NoExceptLibrary, MTLibrary):
     'algorithm.cpp',
     'any.cpp',
     'bind.cpp',
+    'charconv.cpp',
     'chrono.cpp',
     'condition_variable.cpp',
+    'condition_variable_destructor.cpp',
     'debug.cpp',
     'exception.cpp',
-    'future.cpp',
     'functional.cpp',
+    'future.cpp',
     'hash.cpp',
     'ios.cpp',
     'iostream.cpp',
     'locale.cpp',
     'memory.cpp',
     'mutex.cpp',
+    'mutex_destructor.cpp',
     'new.cpp',
     'optional.cpp',
     'random.cpp',
@@ -970,7 +997,7 @@ class libmalloc(MTLibrary, NoBCLibrary):
     return name
 
   def can_use(self):
-    return shared.Settings.MALLOC != 'none'
+    return super(libmalloc, self).can_use() and shared.Settings.MALLOC != 'none'
 
   @classmethod
   def vary_on(cls):
@@ -1060,7 +1087,6 @@ class libgl(MTLibrary):
 
 class libembind(CXXLibrary):
   name = 'libembind'
-  cflags = ['-std=c++11']
   depends = ['libc++abi']
   never_force = True
 
@@ -1112,7 +1138,7 @@ class libasmfs(CXXLibrary, MTLibrary):
   def can_build(self):
     # ASMFS is looking for a maintainer
     # https://github.com/emscripten-core/emscripten/issues/9534
-    return False
+    return True
 
 
 class libhtml5(Library):
@@ -1186,7 +1212,7 @@ class CompilerRTWasmLibrary(Library):
   force_object_files = True
 
   def can_build(self):
-    return shared.Settings.WASM_BACKEND
+    return super(CompilerRTWasmLibrary, self).can_build() and shared.Settings.WASM_BACKEND
 
 
 class libc_rt_wasm(AsanInstrumentedLibrary, CompilerRTWasmLibrary, MuslInternalLibrary):
@@ -1208,10 +1234,10 @@ class libubsan_minimal_rt_wasm(CompilerRTWasmLibrary, MTLibrary):
 class libsanitizer_common_rt_wasm(CompilerRTWasmLibrary, MTLibrary):
   name = 'libsanitizer_common_rt_wasm'
   depends = ['libc++abi']
+  includes = [['system', 'lib', 'libc', 'musl', 'src', 'internal']]
   js_depends = ['memalign', 'emscripten_builtin_memalign', '__data_end', '__heap_base']
   never_force = True
 
-  cflags = ['-std=c++11']
   src_dir = ['system', 'lib', 'compiler-rt', 'lib', 'sanitizer_common']
   src_glob = '*.cc'
   src_glob_exclude = ['sanitizer_common_nolibc.cc']
@@ -1222,7 +1248,6 @@ class SanitizerLibrary(CompilerRTWasmLibrary, MTLibrary):
   never_force = True
 
   includes = [['system', 'lib', 'compiler-rt', 'lib']]
-  cflags = ['-std=c++11']
   src_glob = '*.cc'
 
 
@@ -1309,16 +1334,26 @@ class libstandalonewasm(MuslInternalLibrary):
                    '__month_to_secs.c',
                    '__tm_to_secs.c',
                    '__tz.c',
-                   '__year_to_secs.c'])
+                   '__year_to_secs.c',
+                   'gettimeofday.c',
+                   'localtime.c',
+                   'localtime_r.c',
+                   'gmtime.c',
+                   'gmtime_r.c',
+                   'nanosleep.c',
+                   'mktime.c'])
     # It is more efficient to use JS for __assert_fail, as it avoids always
     # including fprintf etc.
     exit_files = files_in_path(
         path_components=['system', 'lib', 'libc', 'musl', 'src', 'exit'],
         filenames=['assert.c'])
-    return base_files + time_files + exit_files
+    conf_files = files_in_path(
+        path_components=['system', 'lib', 'libc', 'musl', 'src', 'conf'],
+        filenames=['sysconf.c'])
+    return base_files + time_files + exit_files + conf_files
 
   def can_build(self):
-    return shared.Settings.WASM_BACKEND
+    return super(libstandalonewasm, self).can_build() and shared.Settings.WASM_BACKEND
 
 
 # If main() is not in EXPORTED_FUNCTIONS, it may be dce'd out. This can be
@@ -1499,6 +1534,14 @@ def calculate(temp_files, in_temp, stdout_, stderr_, forced=[]):
     force_include.add('libasan_rt_wasm')
     add_library(system_libs_map['libasan_rt_wasm'])
 
+  # the sanitizer runtimes may call mmap, which will need a few things. sadly
+  # the usual deps_info mechanism does not work since we scan only user files
+  # for things, and not libraries (to be able to scan libraries, we'd need to
+  # somehow figure out which of their object files will actually be linked in -
+  # but only lld knows that). so just directly handle that here.
+  if shared.Settings.UBSAN_RUNTIME or shared.Settings.USE_LSAN or shared.Settings.USE_ASAN:
+    shared.Settings.EXPORTED_FUNCTIONS.append(mangle_c_symbol_name('memset'))
+
   if shared.Settings.STANDALONE_WASM:
     add_library(system_libs_map['libstandalonewasm'])
 
@@ -1540,7 +1583,7 @@ class Ports(object):
 
   @staticmethod
   def get_lib_name(name):
-    return shared.static_library_name(name)
+    return name + static_library_ext()
 
   @staticmethod
   def get_include_dir():
@@ -1591,13 +1634,20 @@ class Ports(object):
       commands.append([shared.PYTHON, shared.EMCC, '-c', src, '-O2', '-o', obj, '-w'] + include_commands + flags)
       objects.append(obj)
 
-    run_commands(commands)
+    Ports.run_commands(commands)
     create_lib(output_path, objects)
     return output_path
 
   @staticmethod
-  def run_commands(commands): # make easily available for port objects
-    run_commands(commands)
+  def run_commands(commands):
+    # Runs a sequence of compiler commands, adding importand cflags as defined by get_cflags() so
+    # that the ports are built in the correct configuration.
+    def add_args(cmd):
+      # this must only be called on a standard build command
+      assert cmd[0] == shared.PYTHON and cmd[1] in (shared.EMCC, shared.EMXX)
+      # add standard cflags, but also allow the cmd to override them
+      return cmd[:2] + get_cflags() + cmd[2:]
+    run_build_commands([add_args(c) for c in commands])
 
   @staticmethod
   def create_lib(libname, inputs): # make easily available for port objects
@@ -1775,36 +1825,6 @@ class Ports(object):
     port.clear(Ports, shared)
     shared.try_delete(os.path.join(Ports.get_build_dir(), name))
 
-  @staticmethod
-  def build_native(subdir):
-    shared.Building.ensure_no_emmake('We cannot build the native system library in "%s" when under the influence of emmake/emconfigure. To avoid this, create system dirs beforehand, so they are not auto-built on demand. For example, for binaryen, do "python embuilder.py build binaryen"' % subdir)
-
-    old = os.getcwd()
-
-    try:
-      os.chdir(subdir)
-
-      cmake_build_type = 'Release'
-
-      # Configure
-      check_call(['cmake', '-DCMAKE_BUILD_TYPE=' + cmake_build_type, '.'])
-
-      # Check which CMake generator CMake used so we know which form to pass parameters to make/msbuild/etc. build tool.
-      generator = re.search('CMAKE_GENERATOR:INTERNAL=(.*)$', open('CMakeCache.txt', 'r').read(), re.MULTILINE).group(1)
-
-      # Make variants support '-jX' for number of cores to build, MSBuild does /maxcpucount:X
-      num_cores = str(shared.Building.get_num_cores())
-      make_args = []
-      if 'Makefiles' in generator and 'NMake' not in generator:
-        make_args = ['--', '-j', num_cores]
-      elif 'Visual Studio' in generator:
-        make_args = ['--config', cmake_build_type, '--', '/maxcpucount:' + num_cores]
-
-      # Kick off the build.
-      check_call(['cmake', '--build', '.'] + make_args)
-    finally:
-      os.chdir(old)
-
 
 # get all ports
 def get_ports(settings):
